#include "GRCPrinter.h"
#include "Uml.h"
#include "GRCC.h"
#include "GReATUtils.h"

using namespace GR;
using std::string;



void GRCPrinter::Visit_RootFolder(const RootFolder& rf)
{
// open files
	string cppfilename = _filename + ".cpp";
	string deffilename = _filename + ".def";
	string headerfilename= _filename + ".h";
	string includefilename= _filename + "_inc.h";
	_cppfile.open( cppfilename.c_str());
	_deffile.open( deffilename.c_str());
	_headerfile.open( headerfilename.c_str());
// print #include-s
	_headerfile << "#include \"Uml.h\"\n";
	_deffile << "EXPORTS\n";
	_cppfile << "#include \""+ dropPath( headerfilename) + "\"\n";
	_cppfile << "#include \""+ dropPath( includefilename) + "\"\n";
	_cppfile << "#include \""+ dropPath( _filename+ "_cmptempl.h") + "\"\n";
	_cppfile << "#include <iostream>\t//for std::cout\n";
	_cppfile << "\n#include \"UdmGme.h\"\n";
	_cppfile << "#include \"UdmDom.h\"\n\n";
// init functions
	StrList_t::const_iterator it = _metanames.begin();
	for(;it != _metanames.end(); ++it) {
		const std::string& mn = *it;
		_deffile << "gz__init_" << mn << "\n";

		_cppfile << "bool gz__init_" << mn << "(Uml::Diagram& dgr)\n{\n";
		_cppfile << "\ntry\n{\n"; 
		_cppfile << "\t" << mn << "::Initialize(dgr);\n"; 
		_cppfile << "}\ncatch (const udm_exception& e)\n{\n";
		_cppfile << "\tstd::cout<< std::string(e.what());\n";
		_cppfile << "\treturn true;\n";
		_cppfile << "}\n";
		_cppfile << "\treturn false;\n}\n";

		_headerfile << "extern bool gz__init_" << mn << "(Uml::Diagram& dgr);\n";
	}
// traverse rules
	vector<RewritingRule> rules = rf.RewritingRule_children();
	for(vector<RewritingRule>::iterator itv =rules.begin(); itv!=rules.end(); ++itv) {	
		RewritingRule rule = *itv;
		rule.Accept(*this);
	}
// close files		
	_cppfile.close();
	_deffile.close();
	_headerfile.close();
}
bool GRCPrinter::HasNullReference(const RewritingRule& rule)
{
	set<GR::PatternObject> pObjs = rule.PatternObject_kind_children();
	for(set<GR::PatternObject>::const_iterator i=pObjs.begin(); i!=pObjs.end(); ++i)
	{
		string clsType = (*i).classType();
		if(clsType.empty()) return false;
	}
	return true;
}
void GRCPrinter::Visit_RewritingRule(const RewritingRule& rule)
{
	_rulepath.push_back((string)rule.name());
	const string &rtype = rule.ruleType();
// print rule-s and case-s
	if(rtype == "RULE" || rtype == "CASE") {	
		if(!HasNullReference (rule)) return;
		MakeGuardFunction(rule);
		MakeAttributeMappingFunction(rule);
		MakeGroupFunction(rule);
	}
// traverse rules
	const vector<RuleBase> &subrules = rule.RuleBase_kind_children();
	for(vector<RuleBase>::const_iterator i= subrules.begin(); i!= subrules.end(); ++i)
	{	const RuleBase &rb = *i;
		const string &srtype = rb.type().name();
		if( srtype != "RewritingRule" )
			continue;
		RewritingRule subrule = RewritingRule::Cast(rb);
		subrule.Accept(*this);
	}
	_rulepath.pop_back();
}

bool GRCPrinter::MakeGuardFunction(const RewritingRule& rule)
{	
	const string &guard = rule.guard();
	if(guard == "")
		return false;
	CreateSignature(rule, "Guard", "bool", true);
	_cppfile << "\tbool Gz_guard = true; \n\tbool *guardVal= &Gz_guard;\n\t{\n\t\t";
	_cppfile << rule.guard() << "\n\t}\n\treturn Gz_guard;\n";
	createBlockEnd(true);
	return true;
}

bool GRCPrinter::MakeAttributeMappingFunction(const RewritingRule& rule)
{
	const vector<Code> ams = rule.Code_kind_children();
	if(ams.size()==0)
		return false;
	const set<GR::PatternGroup> groups = rule.PatternGroup_kind_children();
	if(groups.empty())
		CreateSignature(rule, "AM", "void", false);
	else
		CreateGroupAMSignature(rule);

	for(vector<Code>::const_iterator i=ams.begin(); i!=ams.end(); i++) {
		const Code am = *i;
		string code = am.code();
		_cppfile << "\n\t{\n\t\t" << am.code() << "\n\t}\n";
	}
	createBlockEnd(false);
	return true;
}

bool GRCPrinter::MakeGroupFunction(const RewritingRule& rule)
{
	const vector<PatternGroup> groups = rule.PatternGroup_kind_children();
	if(groups.size()==0) return false;

	PatternGroup group = *(groups.begin());
	CreateGroupFuncSignature(rule);

	_cppfile << "\n\t{\n\t\t" << (string)group.rule() << "\n\t}\n";

	createBlockEnd(false);
	return true;
}

void GRCPrinter::createBlockEnd(bool guard)
{
	_cppfile << "}\ncatch (const udm_exception& e)\n{\n";
	_cppfile << "\tstd::cout<< std::string(e.what());\n";
	_cppfile << "\terr = true;\n";
	if (guard)
		_cppfile << "\treturn false;\n";
	_cppfile << "}\n}\n";
}

void GRCPrinter::CreateSignature(const RewritingRule& rule, char *sigvar, char *ret, bool onlyBoundObjects)
{	
	const __int64 &o_id = rule.ruleId();
	StrList_t::iterator si=_rulepath.begin();
	_cppfile << "//======================================\n";
	_cppfile << "//Path : " << *si;
	for(si++;si!=_rulepath.end();si++) {	
		_cppfile << "->" << *si;
	}
	_cppfile << "\n";
	_cppfile << ret << " " << sigvar << "_" << convertToCPlusPlusName( rule.name()) << "_"<< o_id << "( const vector<Udm::Object> &objs, bool& err ) \n{\n" ;
	_headerfile << "extern " << ret << " " << sigvar << "_" << convertToCPlusPlusName( rule.name()) << "_"<< o_id << "( const vector<Udm::Object> &objs, bool& err );\n" ;
	_cppfile << "\ntry\n{\n";
	_cppfile << "\tvector<Udm::Object>::const_iterator obj_iter = objs.begin();\n";

	//Udm::Object obj = *obj_iter;
	_cppfile<< "Udm::Object obj = *obj_iter;\n";
	_cppfile<< "string objname = obj.type().name();\n";

	_deffile << sigvar << "_" << convertToCPlusPlusName( rule.name()) << "_"<< o_id << "\n";

	set<string> names;
	typedef NameCompareFunctor< GR::PatternObject> PatternObjectCompareFunc;
	typedef std::set< GR::PatternObject, PatternObjectCompareFunc> PattObjSorted_t;
	PattObjSorted_t patobjs = rule.PatternObject_kind_children_sorted( PatternObjectCompareFunc());
	PattObjSorted_t::const_iterator i;
	for(i=patobjs.begin();i!=patobjs.end();i++)
	{	const PatternObject &patobj = *i;
		string name = patobj.name();
		// only bound objects support added by VIZA 08/25/04
		const std::string action= patobj.action();
		if ( onlyBoundObjects && ( "CreateNew" == action))
			continue;
		set<string>::iterator ni = names.find(name);
		if(ni!=names.end())
			continue;
		names.insert(name);
		std::string ns_name = patobj.nameSpace();
		std::string typeName = std::string(patobj.paradigmName()) + "::" + std::string(patobj.classType());
		if(!ns_name.empty())
			typeName = std::string(patobj.paradigmName()) + "::" + ns_name + "::" + std::string(patobj.classType());
		_cppfile << "\t" << typeName << " " << patobj.name() << " = ";
		_cppfile << typeName << "::Cast(*obj_iter);\n";
//		_cppfile << "\tUdm::Object " << patobj.name() << " = *obj_iter;\n";
		_cppfile << "\tobj_iter++;\n";
	}
	_cppfile << "\n";
}

void GRCPrinter::CreateGroupAMSignature(const GR::RewritingRule& rule)
{	
	const __int64 &o_id = rule.ruleId();
	StrList_t::iterator si=_rulepath.begin();
	_cppfile << "//======================================\n";
	_cppfile << "//Path : " << *si;
	for(si++;si!=_rulepath.end();si++) {	
		_cppfile << "->" << *si;
	}
	_cppfile << "\n";
	_cppfile << "void AM_" << convertToCPlusPlusName( rule.name()) << "_"<< o_id << "( vector< list<Udm::Object> > &objs, bool& err ) \n{\n" ;
	_headerfile << "extern void AM_" << convertToCPlusPlusName( rule.name()) << "_"<< o_id << "( vector< list<Udm::Object> > &objs, bool& err );\n" ;
	_cppfile << "\ntry\n{\n";
	_cppfile << "\tlist<Udm::Object>::iterator l_iter;\n";
	_cppfile << "\tvector< list<Udm::Object> >::iterator obj_iter = objs.begin();\n";
	_deffile << "AM_" << convertToCPlusPlusName( rule.name()) << "_"<< o_id << "\n";

	set<string> names;
	typedef NameCompareFunctor< GR::PatternObject> PatternObjectCompareFunc;
	typedef std::set< GR::PatternObject, PatternObjectCompareFunc> PattObjSorted_t;
	PattObjSorted_t patobjs = rule.PatternObject_kind_children_sorted( PatternObjectCompareFunc());
	PattObjSorted_t::const_iterator i;
	for(i=patobjs.begin();i!=patobjs.end();i++)
	{	const PatternObject &patobj = *i;
		string name = patobj.name();
		// only bound objects support added by VIZA 08/25/04
		const std::string action= patobj.action();
//		if ( onlyBoundObjects && ( "CreateNew" == action))
//			continue;
		set<string>::iterator ni = names.find(name);
		if(ni!=names.end())
			continue;
		names.insert(name);
	    std::string typeName = std::string(patobj.paradigmName()) + "::" + std::string(patobj.classType());
		std::string objNS = patobj.nameSpace();
		if(!objNS.empty())
			typeName = std::string(patobj.paradigmName()) + "::" + objNS + "::" + std::string(patobj.classType());
		_cppfile << "\tlist<" << typeName << "> " << patobj.name()<< ";\n";
		_cppfile << "\tfor(l_iter=(*obj_iter).begin();l_iter!=(*obj_iter).end();++l_iter)\n";
		_cppfile << "\t\t" << patobj.name() << ".push_back(";
		_cppfile << typeName << "::Cast(*l_iter));\n";
		_cppfile << "\tobj_iter++;\n";
	}
	_cppfile << "\n";
}

void GRCPrinter::CreateGroupFuncSignature(const GR::RewritingRule& rule)
{	
	const __int64 &o_id = rule.ruleId();
	StrList_t::iterator si=_rulepath.begin();
	_cppfile << "//======================================\n";
	_cppfile << "//Path : " << *si;
	for(si++;si!=_rulepath.end();si++) {	
		_cppfile << "->" << *si;
	}
	_cppfile << "\n";
	_cppfile <<"bool Group_" << convertToCPlusPlusName(rule.name()) << "_"<< o_id << "(const vector<Udm::Object> &the_objs, const vector<Udm::Object> &other_objs, bool& err ) \n{\n" ;
	_headerfile << "extern bool Group_" << convertToCPlusPlusName( rule.name()) << "_"<< o_id << "(const vector<Udm::Object> &the_objs, const vector<Udm::Object> &other_objs, bool& err );\n" ;
	_deffile <<"Group_" << convertToCPlusPlusName( rule.name()) << "_"<< o_id << "\n";

	_cppfile << "\ntry\n{\n";
	_cppfile << "\tvector<Udm::Object>::const_iterator the_iter = the_objs.begin();\n";
	_cppfile << "\tvector<Udm::Object>::const_iterator other_iter = other_objs.begin();\n";

	set<string> names;
	typedef NameCompareFunctor< GR::PatternObject> PatternObjectCompareFunc;
	typedef std::set< GR::PatternObject, PatternObjectCompareFunc> PattObjSorted_t;
	PattObjSorted_t patobjs = rule.PatternObject_kind_children_sorted( PatternObjectCompareFunc());
	PattObjSorted_t::const_iterator i;
	for(i=patobjs.begin();i!=patobjs.end();i++)
	{	const PatternObject &patobj = *i;
		string name = patobj.name();
		const std::string action= patobj.action();
		if ( "CreateNew" == action)
			continue;
		set<string>::iterator ni = names.find(name);
		if(ni!=names.end())
			continue;
		names.insert(name);
	    std::string typeName = std::string(patobj.paradigmName()) + "::" + std::string(patobj.classType());
		std::string objNS = patobj.nameSpace();
		if(!objNS.empty())
			typeName = std::string(patobj.paradigmName()) + "::" + objNS + "::" + std::string(patobj.classType());
		_cppfile << "\t" << typeName << " the_" << patobj.name() << " = "<< typeName << "::Cast(*the_iter);\n";
		_cppfile << "\t" << typeName << " other_" << patobj.name() << " = "<< typeName << "::Cast(*other_iter);\n";
		_cppfile << "\tthe_iter++; other_iter++;\n";
	}
	_cppfile << "\n";
}
