#include "GRCforCG.h"
#include "GReATUtils.h"
#include "GRCMPCGen.h"
#include <stdlib.h> // for system()

///////////////////////////////////////////////////////////////////////////////
// GRCC methods
GRCforCG::GRCforCG( const CCodeGenforCGConfig& ccfg) 
	: _ccfg( ccfg)  
{
}

std::string GRCforCG::generateMpc() {
	std::string fnamebase = dropPath( dropExtension( _ccfg._depFileName));
	std::string greatTempDir= getGreatTempDir();
	std::string projfile= greatTempDir + dropPath( _ccfg._genFileName);
	//
	GRCMPCGen* mpcGen= 0;
	if ( _ccfg._useLib) {
		// only static link lib is supported currently;
		mpcGen= new GRCMPCExeUsingStaticLibGen( projfile);
	} else {
		mpcGen= new GRCMPCExeUsingNoLibGen( projfile);
	}
	mpcGen->generate( fnamebase, _ccfg._udmDirName, _ccfg._metas);
	delete mpcGen;
	return projfile+ ".mpc";
}

std::string GRCforCG::generateVisualStudioProject() {
	std::string compileMode = _ccfg._debug ? "\"Win32 Debug\"" : "\"Win32 Release\"";
	std::string mpcFile= "\""+ dropPath( _ccfg._genFileName) + ".mpc\"";
	std::string projtype; // compiler version dependent data
#if _MSC_VER >= 1400
	projtype= "vc8";
#elif _MSC_VER >= 1310
	projtype= "vc71";
#elif _MSC_VER >= 1300
	projtype= "vc7";
#else
	projtype= "vc6";
#endif
	std::string cmdline= "call \"%UDM_3RDPARTY_PATH%\\mpc\\mpc.exe\" -type "+ projtype+ " -into \""+ getPath( _ccfg._genFileName, true)+ "\" "+ mpcFile;
	std::string restoreDir= getCwd();
	changeDir( getGreatTempDir());
	int sysret = system( cmdline.c_str());
	changeDir( restoreDir);
	if ( sysret != 0){	
			throw udm_exception("Error: VS .dsp generation failed.");
	}
	return _ccfg._genFileName+ ".dsp";
}

std::string GRCforCG::generateExe() {
	std::string compileMode = _ccfg._debug ? "\"Win32 Debug\"" : "\"Win32 Release\"";
	std::string mpcFile= "\""+ dropPath( _ccfg._genFileName) + ".mpc\"";
	std::string cmdline= "call \"%GREAT_PATH%\\etc\\GRC_make.bat\"  ";
	cmdline+= mpcFile + " " + "\"Makefile." + convertToMpcName( dropPath( _ccfg._genFileName)) + ".mak\"" + " " + compileMode + " "+ "\"" + getPath( _ccfg._genFileName, true) + "\"";
	std::string restoreDir= getCwd();
	changeDir( getGreatTempDir());
	int sysret = system( cmdline.c_str());
	changeDir( restoreDir);
	if ( sysret != 0){	
			throw udm_exception("Error: Make process of exe failed.");
	}
	std::string exename;
	return exename;
}