#include "Effector.h"
#include "CIntCompiler.h"
#include "Logging.h"
#include "PEStringTable.h"
#include "UmlExt.h"
#include "GReATUtils.h"
#include "OutputSorter.h"

///////////////////////////////////////////////////////////////////////////////
void CodeForAssocRoleFunctor::operator()( const Uml::AssociationRole& role) const
{
	Uml::Association parent= role.parent();
	if ( ( ( std::string) parent.name()).empty())
		return;
	Uml::Class associationClass= parent.assocClass();
	if ( Uml::Class( NULL) == associationClass)
		return;
	Uml::Namespace ns;
	std::string ns_name;
	Udm::Object assocClsParent = associationClass.parent();
	if(std::string(assocClsParent.type().name())=="Namespace")
	{
		ns = Uml::Namespace::Cast(assocClsParent);
		ns_name = ns.name();
	}
//	printLogMsg( "Delete code for association class: "+ ( std::string) parent.name()+  " with role name: "+ ( std::string) role.name()+ ".");
	// Create the definition, which gets the association classes.
	Definition* assocClasses= new Definition;
	//assocClasses->createRoles( _pattern.paradigmName(), 
	//		parent.name(), 
	//		PE::createUniqueName( "del"+ ( std::string)parent.name()+ "s"),
	//		_boundObjects[ _pattern],
	//		role.name()
	//		);
	assocClasses->createRoles( _pattern.paradigmName(), 
			ns_name,
			parent.name(), 
			PE::createUniqueName( "del"+ ( std::string)parent.name()+ "s"),
			_boundObjects[ _pattern],
			role.name()
			);
	_output->add( *assocClasses);
	// Create the loop, which iterates the association classes.
	Loop* assocClassLoop= new Loop;
	assocClassLoop->createIteratorLoop( IT+ assocClasses->_variable, lowerBegin( assocClasses->_variable), assocClasses->_type);
	_output->add( *assocClassLoop);
	// Create current association class definition copying the loop iterator.
	Definition* currAssocClass= new Definition;
	currAssocClass->_type= ( std::string) _pattern.paradigmName()+ RES+ ( std::string) parent.name();
	currAssocClass->_variable= PE::createUniqueName( CURR+ ( std::string)parent.name());
	currAssocClass->_initValue= PTR + assocClassLoop->_loopVariable;
	assocClassLoop->add( *currAssocClass);
	// Delete code for current associtation class.
	_deleteClassFunctor.deleteByName( currAssocClass->_variable, assocClassLoop);
}

///////////////////////////////////////////////////////////////////////////////
bool CodeForDeleteClassFunctor::operator()( const GR::PatternObject& pattern, bool isAssocClass) const
{
	if ( isAssocClass== pattern.isAssociationClass())
	{
		if( "Delete" == ( std::string) pattern.action())
		{
			if ( false== isAssocClass)
				deletePeerAssocClassObjects( pattern);
			deleteByName( _boundObjects[ pattern], _output);
			return true;
		}
	}
	return false;
}

void CodeForDeleteClassFunctor::deleteByName( const std::string& name, PEComposite* addTo) const
{
	// generate code for delete.
	If* objectIsValid= new If;
	objectIsValid->_expression= name;
	Expression* deleteObject= new Expression;
	deleteObject->_expression= lowerBegin( name)+ ".DeleteObject()";
	objectIsValid->add( *deleteObject);
	addTo->add( *objectIsValid);
}

void CodeForDeleteClassFunctor::deletePeerAssocClassObjects( const GR::PatternObject& pattern) const
{
//	Uml::Class patternClass= Uml::classByName( _backend.getDiagram( ( std::string) pattern.paradigmName()), ( std::string)pattern.classType());
	Uml::Class patternClass= Uml::classByName( _backend.getDiagram( ( std::string) pattern.paradigmName()), (std::string)pattern.nameSpace(), ( std::string)pattern.classType());
	set< Uml::AssociationRole> roles= Uml::AncestorAssociationTargetRoles( patternClass);
	for_each( roles.begin(), roles.end(), CodeForAssocRoleFunctor( _boundObjects, *this, pattern, _output));
}

///////////////////////////////////////////////////////////////////////////////
void Effector::executeRightSide( PEComposite& code)
{
	_currCode= &code;
	createMatchLoop();
	createObjects();
	_cc.init( _rule, _boundObjects, Class() /*unused*/);
	if ( true== _cc.isCode())
		_cc.createCode( *_currCode);
	bindRemainingObjectLinks();
	// Remove all association class with action: 'Delete'.
	deleteAssocClassObjects();
	// Remove all other objects with action: 'Delete'.
	deleteObjects();
}

void Effector::createObjects()
{
	// Create parent-child compositions.
	GR::ObjectWrapper parent;
	GR::ObjectWrapper child;
	std::string childRole;
	while ( true== getCompositionToCreate( parent, child, childRole))
		createOutputObject( parent, child, childRole);
}

void Effector::bindRemainingObjectLinks()
{
	set< GR::ObjectLink> links= _rule.ObjectLink_kind_children();
	for( set< GR::ObjectLink>::const_iterator itLinks= links.begin(); itLinks!= links.end(); ++itLinks)
	{
		const GR::ObjectLink& currLink= *itLinks;
		if ( "CreateNew"== ( std::string) currLink.action())
			createNewObjectLink( currLink);
		if ( "Delete"== ( std::string) currLink.action())
			deleteObjectLink( currLink);
	}
}

bool Effector::getCompositionToCreate( GR::ObjectWrapper& parentOW, GR::ObjectWrapper& childOW, std::string& childRole) const
{
	set< GR::ObjectLink> links= _rule.ObjectLink_kind_children();
	for( set< GR::ObjectLink>::const_iterator itLinks= links.begin(); itLinks!= links.end(); ++itLinks)
	{
		const GR::ObjectLink& currLink= *itLinks;
		if ( "Composition" != ( std::string)currLink.relationshipType())
			continue;
		GR::ObjectWrapper srcOW= currLink.src_end();
		GR::ObjectWrapper dstOW= currLink.dst_end();
		// Composition to create?		
		if( srcOW==Udm::null || dstOW==Udm::null)
			throw CGEx("FATAL: Invalid composition relationship found.");
		if( ( 0!= dstOW.objectID()) && ( 0== srcOW.objectID()))
		{
			if ( ( "CreateNew"!= ( std::string)srcOW.action()) || ( "CreateNew"!= ( std::string)currLink.action()))
				throw CGEx("FATAL: Invalid composition relationship found in Effector.");
			parentOW= dstOW;
			childOW= srcOW;
			childRole= currLink.srcRoleName();	// child role is represented as source role name
//			printLogMsg("Composition to create found. Parent: " + ( std::string)parentOW.name() + " Child: " + ( std::string)childOW.name());
			return true;
		}
	}
	return false;
}

void Effector::createOutputObject( const GR::ObjectWrapper& parentOW, GR::ObjectWrapper& childOW, const std::string& childRole)
{
	Definition* newChild= new Definition;
	/*newChild->createUdmCreate( ( std::string)childOW.paradigmName(), 
		( std::string)childOW.classType(), 
		lowerBegin( PE::createUniqueName( "new"+ ( std::string)childOW.name())), 
		lowerBegin( _boundObjects[ parentOW]),
		childRole.empty() ? "" : ( std::string)parentOW.paradigmName()+ RES+ ( std::string)parentOW.classType()+ RES+ "meta_"+ childRole
		);*/
	newChild->createUdmCreate( PE::getFullType(childOW),
		lowerBegin( PE::createUniqueName( "new"+ ( std::string)childOW.name())), 
		lowerBegin( _boundObjects[ parentOW]),
		childRole.empty() ? "" : PE::getFullType(parentOW) + RES+ "meta_"+ childRole
		);
	_currCode->add( *newChild);
	// bind child
	childOW.objectID()= 1;
	_boundObjects[ childOW]= newChild->_variable;
}

void Effector::deleteAssocClassObjects()
{	
	set< GR::PatternObject> patterns= _rule.PatternObject_kind_children();
	for_each( patterns.begin(), patterns.end(), std::bind2nd( CodeForDeleteClassFunctor( _boundObjects, _backend, _currCode), true));
}

void Effector::deleteObjects()
{	
	set< GR::PatternObject> patterns= _rule.PatternObject_kind_children();
	for_each( patterns.begin(), patterns.end(), std::bind2nd( CodeForDeleteClassFunctor( _boundObjects, _backend, _currCode), false));
}

void Effector::createNewObjectLink( const GR::ObjectLink& linkToCreate)
{
	if( "Composition"== ( std::string) linkToCreate.relationshipType())
		return;	// compositions are processed already, see 'getCompositionToCreate()'.
	GR::ObjectWrapper srcOW= linkToCreate.src_end();
	GR::ObjectWrapper dstOW= linkToCreate.dst_end();
	std::string linkType= linkToCreate.relationshipType();
	// Create code.
	if ( srcOW.isAssociationClass() && ( linkType=="AssociationClass"))
	{
		Definition* assignDstEnd= new Definition;
		assignDstEnd->_variable= lowerBegin( _boundObjects[ srcOW])+ "."+ ( std::string)linkToCreate.dstRoleName()+ "_end()";
		assignDstEnd->_initValue= lowerBegin( _boundObjects[ dstOW]);
		_currCode->add( *assignDstEnd);
	}
	else if  ( dstOW.isAssociationClass() && ( linkType=="AssociationClass"))
	{
		Definition* assignSrcEnd= new Definition;
		assignSrcEnd->_variable= lowerBegin( _boundObjects[ dstOW])+ "."+ ( std::string)linkToCreate.srcRoleName()+ "_end()";
		assignSrcEnd->_initValue= lowerBegin( _boundObjects[ srcOW]);
		_currCode->add( *assignSrcEnd);
	}
	else // none of srcOW and dstOW are association classes.
	{
		bool isMulCardinality= ( 1 < 
			(unsigned int) getAssociationCardinality( ( std::string)srcOW.paradigmName(), (std::string)srcOW.nameSpace(), ( std::string)srcOW.classType(), ( std::string)linkToCreate.dstRoleName(), false));
		Expression* newLink= new Expression;
		newLink->_expression= lowerBegin( _boundObjects[ srcOW])+ "."+ ( std::string)linkToCreate.dstRoleName()+ "()"+
			(isMulCardinality ? PEQ : EQ)+ SEP+ lowerBegin( _boundObjects[ dstOW]);
		_currCode->add( *newLink);
	}
}

void Effector::deleteObjectLink( const GR::ObjectLink& linkToDelete)
{
	if( "AssociationClass"== ( std::string)linkToDelete.relationshipType())
	{
		return;	// deleteObjectLink() is now called before delete(Association)Objects, so "AssociationClass" links with 
		// action "Delete" can exist still here. ->This is not an error anymore..

		//x //!!VIZA this can never happen, because association classes can exist only with links.
		//x //if link is to delete, association class is to delete, which is already done in deleteAssocClass.
		//x throw CGEx("FATAL: CG inconsistent state. Association class link delete without deleting association class.");
	}
	if ( "Composition" == ( std::string) linkToDelete.relationshipType())
	{
		// check if child's action is set to 'delete'.
		GR::ObjectWrapper srcOW= linkToDelete.src_end();
		if ( "Delete" == ( std::string)srcOW.action())
			return;	// OK, delete code already generated in deleteObjects().
		else
			throw CGEx("FATAL: CG inconsistent state. Composition link delete without deleting child.");
	}
	GR::ObjectWrapper srcOW= linkToDelete.src_end();
	GR::ObjectWrapper dstOW= linkToDelete.dst_end();
//	if( ( "Delete"== ( std::string) srcOW.action()) || ( "Delete"== ( std::string) dstOW.action()))
//		throw CGEx("Invalid effector scenario in deleteObjectLink().");	// !!VIZA can this ever happen?
	if ( srcOW.isAssociationClass() || dstOW.isAssociationClass())
		throw CGEx("FATAL: Association class can be connected only with association class link!");
	else
	{
		// So this is surely simple association :)
		bool isMulCardinality= ( 1 < 
			(unsigned int) getAssociationCardinality( ( std::string)srcOW.paradigmName(), (std::string)srcOW.nameSpace(), ( std::string)srcOW.classType(),  ( std::string)linkToDelete.dstRoleName(),  srcOW.isAssociationClass() ));
		Expression* deleteLink= new Expression;
		deleteLink->_expression= lowerBegin( _boundObjects[ srcOW])+ "."+ ( std::string)linkToDelete.dstRoleName()+ "()"+
			(isMulCardinality ? MEQ+ SEP+ lowerBegin( _boundObjects[ dstOW]) : EQ+ SEP+ "Udm::null");
		_currCode->add( *deleteLink);
	}
}

void Effector::createMatchLoop()
{
	Loop* matchLoop= new Loop;
	matchLoop->createMatchLoop();
	_currCode->add( *matchLoop);
	_currCode= matchLoop;
}

/*!!VIZA this is temporary, and should be replace by the corrected one in UmlExt.cpp*/
/*
static set< Uml::AssociationRole> AncestorCrossAssociationTargetRoles(const Uml::Class &c, const Uml::Diagram & cross_dgr)
{
	using namespace Uml;
	set<AssociationRole> ret;
	set<Uml::Class> anc_cs = AncestorClasses(c);
	set<Uml::Class>::iterator i = anc_cs.begin();
	while (i != anc_cs.end())
	{
		Uml::Class a_i = *i;
		Uml::Class ph_a_i = classByName(cross_dgr, a_i.name());
		std::string a_i_name= a_i.name();
		if ( ph_a_i)
		{
			std::string name= ph_a_i.name();
			set<AssociationRole> ret_i = AssociationTargetRoles(ph_a_i);
			int size_ret_i= ret_i.size();
			ret.insert(ret_i.begin(), ret_i.end());
		}
		i++;
	};

	return ret;
}
*/

extern bool getAssociationProperties( 
	const GeneralBackend& backend,
	const std::string& paradigm, 
	const std::string& boundclass, 
	const std::string& association, 
	bool isSourceAssoc,
	Uml::Class& targetClass, 
	Uml::Association& assocClass,
	int& maxMultiplicity,
	bool& isCrossLink
	);

extern bool getAssociationProperties( 
	const GeneralBackend& backend,
	const std::string& paradigm, 
	const std::string& ns_name,
	const std::string& boundclass, 
	const std::string& association, 
	bool isSourceAssoc,
	Uml::Class& targetClass, 
	Uml::Association& assocClass,
	int& maxMultiplicity,
	bool& isCrossLink
	);

int Effector::getAssociationCardinality( 
	const std::string& paradigm, 
	const std::string& boundclass, 
	const std::string& association, 
	bool isSourceAssoc
	) const
{
	Uml::Class targetClass;
	Uml::Association assocClass;
	int maxMultiplicity= 0;
	bool isCrossLink= false;
	bool found= getAssociationProperties( _backend, paradigm, boundclass, association, isSourceAssoc, targetClass, assocClass, maxMultiplicity, isCrossLink);
	if ( found) {
		return maxMultiplicity;
	}
	// Not found, or exception..
	printError("Association role: "+ association+" not found for"+ ( isSourceAssoc ? " association" : "")+ " class: "+ boundclass+ ".");
	return 0;
}

int Effector::getAssociationCardinality( 
	const std::string& paradigm, 
	const std::string& ns_name,
	const std::string& boundclass, 
	const std::string& association, 
	bool isSourceAssoc
	) const
{
	Uml::Class targetClass;
	Uml::Association assocClass;
	int maxMultiplicity= 0;
	bool isCrossLink= false;
	bool found= getAssociationProperties( _backend, paradigm, ns_name, boundclass, association, isSourceAssoc, targetClass, assocClass, maxMultiplicity, isCrossLink);
	if ( found) {
		return maxMultiplicity;
	}
	// Not found, or exception..
	printError("Association role: "+ association+" not found for"+ ( isSourceAssoc ? " association" : "")+ " class: "+ boundclass+ ".");
	return 0;
}

//to-do list
//1. move: set simple link