/* NAME:	Output appender
 * AUTHOR:	Attila Vizhanyo - viza@isis-server.isis.vanderbilt.edu
 *
 * Generate code, which appends output with a packet
 */
#ifndef __OUTPUTAPPENDER_H
#define __OUTPUTAPPENDER_H

#ifdef _MSC_VER
#pragma warning (disable : 4786)
#pragma warning (disable : 4503) 
#endif

#include <map>
#include "RuleTypes.h"
#include "PE.h"
#include "PacketPassing.h"

// This class is not designed to be a base class. 
// Either don't inherit from it, or add virtual destructor.
class OutputAppender
{
public:
	// ctor; see parameter desc. at the member variables declaration part. 
	OutputAppender( const GRSimpleRule& rule, 
		const std::map< GR::ObjectWrapper, std::string>& boundObjects
		) : _rule( rule) , _boundObjects( boundObjects)
		{}
	// Generate output appender code for 'ruleOutputs' which add the outputs 'ruleOutputs' to the the output member variables,
	// into 'filterFuncDef' and 'filterFuncCall'. 
	void addOutputs( const ParamDatas_t& ruleOutputs, Function& filterFuncDef, FunctionCall& filterFuncCall);

protected:
	// Return with generated function parameter 'param' for 'output'.
	void createParam( const ParamData& output, FuncParam_t& param) const;

private:
	// Rule, on which output filter code generation executes.
	GRSimpleRule _rule;
	// Maps object wrappers to their generated variable names. 
	const std::map< GR::ObjectWrapper, std::string>& _boundObjects;

};

#endif //__OUTPUTAPPENDER_H