#include "PE.h"
#include <algorithm>
#include "GReATUtils.h"
#include "PEStringTable.h"

using namespace std;
// PE static data members initialization.
unsigned short PE::_tab= 0;
unsigned long PE::_id= 0;


// operator implementations
bool operator==( const FuncParam_t& cmp1, const FuncParam_t& cmp2)
{
	if ( cmp1._param.first == cmp2._param.first)
		return true;
	return false;
}

std::ostream& operator<<( std::ostream& os, const PE& pe)
{
	return pe.print( os); // calls the appropriate print
}

// PE static functions
void PE::resetPE_UID()
{
	_id= 0;
}

std::string PE::getPE_UID()
{
	return std::string( "_")+ longHexToString( _id++);
};

std::string PE::createUniqueName( const std::string& name, bool decorateName)
{
	return name+ ( decorateName ? getPE_UID() : "");
}

std::string PE::getFullType(const GR::ObjectWrapper &grObj)
{
	std::string ns = grObj.nameSpace();
	if(ns.empty()) 
	{
		return (std::string)(grObj.paradigmName())+RES+(std::string)(grObj.classType());
	}
	else
		return (std::string)(grObj.paradigmName()) + RES + ns + RES + (std::string)(grObj.classType());
}

std::ostream& PE::printTab( std::ostream& os)
{
	for( int i=0; i<_tab; ++i)
		os << "\t";
	return os;
}

// print
std::ostream& PEComposite::print( std::ostream &os) const
{
	for( PEs_t::const_iterator itPrint= _elems.begin(); itPrint != _elems.end(); ++itPrint)
	{
		const PE* pe= (*itPrint);
		//os= pe->print( os);
		pe->print( os);
	}
	return os;
}

std::ostream& PEComposite::printHeader( std::ostream &os) const
{
	for( PEs_t::const_iterator itPrint= _elems.begin(); itPrint != _elems.end(); ++itPrint)
	{
		const PE* pe= (*itPrint);
		//os= pe->printHeader( os);
		pe->printHeader( os);
	}
	return os;
}

std::ostream& PEComposite::printSource( std::ostream &os) const
{
	for( PEs_t::const_iterator itPrint= _elems.begin(); itPrint != _elems.end(); ++itPrint)
	{
		const PE* pe= (*itPrint);
		pe->printSource( os);
	}
	return os;
}

std::ostream& Function::print( std::ostream &os) const
{
	printSignature( os, true);
	os << std::endl;
	printBody( os);
	os << endl;
	return os;
}

std::ostream& Function::printHeader( std::ostream &os) const
{
	printSignature( os, false);
	os << ";" << std::endl;
	return os;

}

std::ostream& Function::printSignature( std::ostream &os, bool printScope) const
{
	printTab( os);
	// print signature
	os << ( ( std::string( "") == _returnValue) ? std::string( "void") : _returnValue) << SEP;
	if ( printScope && ( false== _scope.empty()))
		os << _scope << RES;
	os << lowerBegin( _name) << "(";
	FuncParams_t::const_iterator itPrint= _params.begin(); 
	while ( itPrint != _params.end())
	{
		os << SEP << (*itPrint)._param.first << SEP << (*itPrint)._param.second;
		++itPrint;
		// put comma, if not last parameter
		if ( itPrint != _params.end())
			os << ",";
	}
	os << ")";
	return os;
}

std::ostream& Function::printBody( std::ostream &os) const
{
	// print body
	printTab( os);
	os << "{" << endl;
	++_tab;
	PEComposite::print( os);
	--_tab;
	printTab( os);
	os << "}" << endl;
	return os;
}

std::ostream& Loop::print( std::ostream &os) const
{
	printTab( os);
	// print loop expression
	os << "for(" << SEP << _from << ";" << SEP << _to << ";" << SEP << _increment << ")" << endl;
	// print loop body
	printTab( os);
	os << "{" << endl;
	++_tab;
	PEComposite::print( os);
	--_tab;
	printTab( os);
	os << "}" << endl;
	return os;
}

std::ostream& While::print( std::ostream &os) const
{
	printTab( os);
	// print loop expression
	os << "while(" <<_condition <<")" << endl;
	// print loop body
	printTab( os);
	os << "{" << endl;
	++_tab;
	PEComposite::print( os);
	--_tab;
	printTab( os);
	os << "}" << endl;
	return os;
}
std::ostream& If::print( std::ostream &os) const
{
	printTab( os);
	// print loop expression
	os << "if(" << SEP << _expression << ")" << endl;
	// print loop body
	printBranch( os, *this);
	if ( 0< sizeElse())
	{
		printTab( os);
		os << "else" << endl;
		printBranch( os, _elseBranch);
	}
	return os;
}

std::ostream& If::printBranch( std::ostream& os, const PEComposite& branch) const
{
	// print loop body
	if ( 1 != branch.size())
	{
		printTab( os);
		os << "{" << endl;
	}
	++_tab;
	if ( &branch == this)
		PEComposite::print( os);
	else
		branch.print( os);
	--_tab;
	if ( 1 != branch.size())
	{
		printTab( os);
		os << "}" << endl;
	}
	return os;
}

std::ostream& Case::print( std::ostream &os) const
{
	printTab( os);
	os << "case" << SEP << _value << ":" << endl;
	++_tab;
	PEComposite::print( os);
	if ( _break)
	{
		printTab( os);
		os << "break;" << endl;
	}
	--_tab;
	return os;
}

std::ostream& Switch::print( std::ostream &os) const
{
	if ( 0 == size())
		return os;	// nothing to print
	// print switch 
	printTab( os);
	os << "switch(" << SEP << _switchVariable << ")" << endl;
	// print switch body
	printTab( os);
	os << "{" << endl;
	++_tab;
	PEComposite::print( os);
	--_tab;
	printTab( os);
	os << "}" << endl;
	return os;
}

std::ostream& ClassDef::print( std::ostream &os) const
{
	printTab( os);
	if ( !_type.empty())
		os << _type << SEP;
	os << lowerBegin( _variable);
	os << ";" << endl;
	return os;
}

std::ostream& Definition::print( std::ostream &os) const
{
	// ClassDef::print() not used because of the trailing semicolon is printed.
	printTab( os);
	if ( !_type.empty())
		os << _type << SEP;
	os << lowerBegin( _variable);
	if ( 0< _args.size())
	{
		os << "(";
		list< std::string>::const_iterator itPrint= _args.begin(); 
		while ( itPrint != _args.end())
		{
			os << SEP << (*itPrint);
			++itPrint;
			// put comma, if not last parameter
			if ( itPrint != _args.end())
				os << ",";
		}
		os << ")";
	}
	if ( !_initValue.empty())
		os << "=" << SEP << _initValue;
	os << ";" << endl;
	return os;
}


std::ostream& Expression::print( std::ostream& os) const
{
	printTab( os);
	os << _expression << ";" << endl;
	return os;
}

std::ostream& CommentLine::print( std::ostream& os) const
{
	printTab( os);
	os << "//" << SEP << _comment << endl;
	return os;
}

std::ostream& Macro::print( std::ostream& os) const
{
	if ( _defVar.empty())
		return os;
	os << "#ifdef" << SEP << _defVar << endl; // no printTab
	PEComposite::print( os);
	os << "#endif" << endl;
	return os;
}

std::ostream& Class::print( std::ostream &os) const
{
	printTab( os);
	// print class header
	os << "class" << SEP << _name << endl;
	printTab( os);
	os << "{" << endl;
	// print public
	if ( 0< _public.size())
	{
		printTab( os);
		os << "public:" << endl;
		++_tab;
		_public.print( os);
		--_tab;
	}
	// print protected
	if ( 0< _protected.size())
	{
		os << endl;
		printTab( os);
		os << "protected:" << endl;
		++_tab;
		_protected.print( os);
		--_tab;
	}
	// print private
	if ( 0< _private.size())
	{
		os << endl;
		printTab( os);
		os << "private:" << endl;
		++_tab;
		 _private.print( os);
		--_tab;
	}
	printTab( os);
	os << "};" << endl<< endl;
	return os;
}

std::ostream& Class::printHeader( std::ostream& os) const
{
	printTab( os);
	// print class header
	os << "class" << SEP << _name << endl;
	printTab( os);
	os << "{" << endl;
	// print public
	if ( 0< _public.size())
	{
		printTab( os);
		os << "public:" << endl;
		++_tab;
		_public.printHeader( os);
		--_tab;
	}
	// print protected
	if ( 0< _protected.size())
	{
		os << endl;
		printTab( os);
		os << "protected:" << endl;
		++_tab;
		_protected.printHeader( os);
		--_tab;
	}
	// print private
	if ( 0< _private.size())
	{
		os << endl;
		printTab( os);
		os << "private:" << endl;
		++_tab;
		_private.printHeader( os);
		--_tab;
	}
	printTab( os);
	os << "};" << endl<< endl;
	return os;
}

std::ostream& Class::printSource( std::ostream& os) const
{
	// print public
	_public.printSource( os);
	// print protected
	_protected.printSource( os);
	// print private
	_private.printSource( os);
	return os;
}

std::ostream& Class::printFwdDecl( std::ostream& os) const
{
	printTab( os);
	// print class header
	os << "class" << SEP << _name << ";" << std::endl;
	return os;
}

std::ostream& Call::printCall( const std::string& name, std::ostream& os) const
{
	printTab( os);
	if ( !_retType.empty())
		os << _retType << SEP;
	if ( !_retVar.empty())
		os << _retVar << EQ << SEP;
	os << lowerBegin( name) << "(" << SEP;
	CallParams_t::const_iterator itPrint= _params.begin(); 
	while ( itPrint != _params.end())
	{
//		os << lowerBegin( *itPrint);
		os << *itPrint;
		++itPrint;
		// put comma, if not last parameter
		if ( itPrint != _params.end())
			os << "," << SEP;
	}
	os << ");" << endl;
	return os;
}

std::ostream& FunctionCall::print( std::ostream &os) const
{
	if ( NULL== _func)
		return os;  // print nothing
	return printCall( _func->_name, os);
}

std::ostream& FuncOperator::print( std::ostream &os) const
{
	if ( NULL== _class)
		return os;	// print nothing
	printTab( os);
	os << _class->_name << SEP;
	std::string objName= ( _name.empty()) ? lowerBegin( _class->_name) : _name;
	os << objName << ";" << endl;
	return printCall( objName, os);
}

std::ostream& TryCatchBlock::print( std::ostream &os) const
{
	printTab( os);
	// print try
	os << "try" << endl;
	// print try body
	printBlock( os, *this);
	// print catch
	printTab( os);
	os << "catch(" << SEP << _catchExpr << ")" << endl;
	printBlock( os, _catchBlock);
	return os;
}

std::ostream& TryCatchBlock::printBlock( std::ostream& os, const PEComposite& block) const
{
	// print loop body
	printTab( os);
	os << "{" << endl;
	++_tab;
	if ( &block == this)
		PEComposite::print( os);
	else
		block.print( os);
	--_tab;
	printTab( os);
	os << "}" << endl;
	return os;
}

std::ostream& Program::print( std::ostream &os) const
{
	PEComposite::print( os);
	for( Classes_t::const_iterator it= _classes.begin(); it!= _classes.end(); ++it)
		(*it)->print( os);
	return os;
}

std::ostream& Program::printSource( std::ostream &os) const
{
	PEComposite::printSource( os);
	for( Classes_t::const_iterator it= _classes.begin(); it!= _classes.end(); ++it)
		(*it)->printSource( os);
	return os;
}

std::ostream& Program::printHeader( std::ostream& os) const
{
	os << std::endl << "// Forward declarations." << std::endl;
	for( Classes_t::const_iterator it= _classes.begin(); it!= _classes.end(); ++it)
		(*it)->printFwdDecl( os);
	os << std::endl << "// Class declarations." << std::endl;
	for( Classes_t::const_iterator it2= _classes.begin(); it2!= _classes.end(); ++it2)
		(*it2)->printHeader( os);
	return os;
}

// Function misc. functions
FuncParam_t Function::createParam( const std::string& namesp, const std::string& type, const std::string& name, bool decorateName)
{
	std::string paramType;
	if ( false == namesp.empty())
		paramType+= namesp+ RES;
	paramType+= type+ REF;
	std::string paramName=  createUniqueName( name, decorateName);
	FuncParam_t param;
	param._param= make_pair( paramType, paramName);
	return param;
}

FuncParam_t Function::createInputParam( const std::string& name, bool decorateName)
{
	return createParam( "", CST+ SEP+ PKS, name, decorateName);
}

FuncParam_t Function::createOutputParam( const std::string& name, bool decorateName)
{
	return createParam( "", PKS, name, decorateName);
}

const FuncParam_t& Function::insertInputParam( const FuncParam_t& inputParam)
{
	FuncParam_t outParamType= createOutputParam( "");
	FuncParams_t::iterator itFind= std::find( _params.begin(), _params.end(), outParamType);
	return *( _params.insert( itFind, inputParam));
}

const FuncParam_t& Function::insertOutputParam( const FuncParam_t& outputParam)
{
	return *( _params.insert( _params.end(), outputParam));
}

// Loop misc. functions
void Loop::addIterator( const std::string& iter, const std::string& container, const std::string& containerType, bool isContainerPointer, bool isConstIterator)
{
	if ( true== _isFirstIterator)
	{
		if(isConstIterator) _from= containerType+ RES+ CIT+ SEP;
		else _from= containerType+ RES+ "iterator"+ SEP;
		_to= "";
		_increment= "";
	} else
	{
		_from+= ","+ SEP;
		_to+= ","+ SEP;
		_increment+= "," + SEP;
	}
	_loopVariable= iter;
	_from+= iter+ EQ+ SEP+ lowerBegin( container)+ ( isContainerPointer ? ACC : ".") + BEG;
	_to+= iter+ NEQ+ SEP+ lowerBegin( container)+ ( isContainerPointer ? ACC : ".") + END;
	_increment+= INC+ iter;
	// from now on subsequent iterators will be generated differently.
	_isFirstIterator= false;
}

void Loop::addInputIterator( const std::string& iter, const std::string& container, bool isContainerPointer)
{
	addIterator( iter, container, PKS, isContainerPointer);
}

void Loop::createIteratorLoop( const std::string& iter, const std::string& container, const std::string& containerType, bool isContainerPointer, bool isConstIterator)
{
	// initialize
	_isFirstIterator= true;
	addIterator( iter, container, containerType, isContainerPointer, isConstIterator);
}

void Loop::createMatchLoop()
{
	createIteratorLoop( IT+ MATCH, _MATCHES, "std::list<"+ SEP+ MATCH+ ">");
	Definition* currMatch= new Definition;
//	currMatch->_type= CST+ SEP+ MATCH+ REF;
	currMatch->_type= MATCH;
	currMatch->_variable= CURRM;
	currMatch->_initValue= PTR+ _loopVariable;
	add( *currMatch);
}

// If misc. functions
void If::appendWithAnd( const std::string& expressionTag)
{
	if ( isExpression())
		_expression+= "&&"+ SEP;
	_expression+= "(" + SEP+ expressionTag+ ")";
}

void If::appendWithOr( const std::string& expressionTag)
{
	if ( isExpression())
		_expression+= "||"+ SEP;
	_expression+= "(" + SEP+ expressionTag+ ")";
}

bool If::isExpression() const
{
	return !_expression.empty();
}

void If::createIfTypeMismatchThenContinue( const std::string& varName, const std::string& varParadigm, const std::string& varType)
{
	createIfExprThenContinue( 
		FAL+ EQEQ+ SEP+ "Uml::IsDerivedFrom("+ SEP+ varName+ ".type(),"+ SEP+ varParadigm+ RES+ varType+ RES+ "meta)"
		);
}

void If::createIfTypeMismatchThenContinue( const std::string& varName, const std::string& varParadigm, const std::string& varNS, const std::string& varType)
{
	if (varNS.length() == 0)
		createIfTypeMismatchThenContinue(varName, varParadigm, varType);
	else
		createIfExprThenContinue( 
		FAL+ EQEQ+ SEP+ "Uml::IsDerivedFrom("+ SEP+ varName+ ".type(),"+ SEP+ varParadigm+ RES+ varNS+ RES+ varType+ RES+ "meta)"
		);
}

void If::createIfExprThenReturn( const std::string& expr)
{
	_expression= expr;
	add( *new Expression( RET));
}

void If::createIfExprThenContinue( const std::string& expr)
{
	_expression= expr;
	add( *new Expression( "continue"));
}

void If::createHitCheckCondition( const std::string& outputMemberName, const std::string& findName)
{
	_expression= outputMemberName+ "."+ END+ NEQ+ SEP+ findName;
	Definition* unique= new Definition;
	unique->_variable= UNQ;
	unique->_initValue= TRU;
	addElse( *unique);
}

// ClassDef misc. functions
void ClassDef::createPackets( const std::string& name)
{
	_type= PKS;
	_variable= name;
}

void ClassDef::createPacketsPtr( const std::string& name)
{
	_type= PKS+ PTR;
	_variable= name;
}

// Definition misc. functions
void Definition::createContainmentDef( const std::string& paradigm, const std::string& type, const std::string& variable, const std::string& parent)
{
	_type= "set<"+ SEP+ paradigm+ RES+ type+ ">";
	_variable= variable;
	_initValue= parent+ "."+ type+ "_kind_children()";
}

void Definition::createContainmentDef( const std::string& paradigm, const std::string& ns_name, const std::string& type, const std::string& variable, const std::string& parent, bool samens)
{
//	_type= "set<"+ SEP+ paradigm+ RES+ type+ ">";
	if(ns_name.length() == 0)
		createContainmentDef(paradigm, type, variable, parent);
	else
	{
		_type= "set<"+ SEP+ paradigm+ RES+ ns_name+ RES + type+ ">";
		_variable= variable;
		if(samens)
			_initValue= parent+ "."+type+ "_kind_children()";
		else
		{
			string ns_path = ns_name;
			string delim("::");
			string::size_type i = ns_path.find(delim);
			while (i != string::npos)
			{
				ns_path.replace(i, i+1, "_");
				i = ns_path.find(delim);
			}
			_initValue= parent+ "."+ ns_path+"_"+type+ "_children()";
		}
	}
}

void Definition::createCurrentInput( const std::string& currInputVar, const std::string& iter)
{
	_type= CST+ SEP+ "Udm::Object"+ REF;
	_variable= currInputVar;
	_initValue= PTR+ iter;
}

void Definition::createRoles( const std::string& paradigm, const std::string& type, const std::string& variable, const std::string& parent, const std::string& role)
{
	_type= "set<"+ SEP+ paradigm+ RES+ type+ ">";
	_variable= variable;
	_initValue= parent+ "."+ role+ "()";
}

void Definition::createRoles( const std::string& paradigm, const std::string& ns_name, const std::string& type, const std::string& variable, const std::string& parent, const std::string& role)
{
	if(ns_name.length() == 0)
		_type= "set<"+ SEP+ paradigm+ RES+ type+ ">";
	else
		_type= "set<"+ SEP+ paradigm+ RES+ ns_name+ RES + type+ ">";
	_variable= variable;
	_initValue= parent+ "."+ role+ "()";
}
void Definition::createRole( const std::string& paradigm, const std::string& type, const std::string& variable, const std::string& parent, const std::string& role)
{
	_type= paradigm+ RES+ type;
	_variable= variable;
	_initValue= parent+ "."+ role+ "()";
}

void Definition::createRole( const std::string& paradigm, const std::string& ns_name, const std::string& type, const std::string& variable, const std::string& parent, const std::string& role)
{
	if(ns_name.length() == 0)
		return createRole(paradigm,type,variable,parent,role);
	_type= paradigm+ RES+ ns_name+RES+type;
	_variable= variable;
	_initValue= parent+ "."+ role+ "()";
}

void Definition::createTypeCast( const std::string& type, const std::string& variable, const std::string& varToCast)
{
	_type= type;
	_variable= variable;
	_initValue= type+ RES+ "Cast("+ SEP+ varToCast+ ")";
}

void Definition::createAssocInfo( const std::string& variable)
{
	_type= "Udm"+ RES+ "Object"+ RES+ "AssociationInfo";
	_variable= variable;
	addArg( "Uml::Class( NULL)");
}

void Definition::createUdmCreate( const std::string& paradigm, const std::string& type, const std::string& variable, const std::string& parent, const std::string& childRole)
{
	_type= paradigm+ RES+ type;
	_variable= variable;
	_initValue=  _type+ RES+ "Create("+ SEP+ parent+ ( childRole.empty() ? "" : (","+ SEP+ childRole) )+ ")";
}

void Definition::createUdmCreate( const std::string& fullType, const std::string& variable, const std::string& parent, const std::string& childRole)
{
	_type= fullType;
	_variable= variable;
	_initValue=  _type+ RES+ "Create("+ SEP+ parent+ ( childRole.empty() ? "" : (","+ SEP+ childRole) )+ ")";
}
// FunctionCall misc. functions
// deprecated
void FunctionCall::copyParamNames()
{
	if ( NULL == _func)
		return;
	for( FuncParams_t::const_iterator itParams= _func->_params.begin(); itParams!= _func->_params.end(); ++ itParams)
	{
		std::string nameParam= (*itParams)._param.second;
		_params.push_back( nameParam);	
	}
}
