#ifndef COPYTOMEMDNCODEPRINTER_H
#define COPYTOMEMDNCODEPRINTER_H

#include "DNProjectCodePrinter.h"

class CopyToMemDNCodePrinter : public DNCodePrinter
{
public:
	CopyToMemDNCodePrinter() : DNCodePrinter() , _memDNDef( 0), _memDNCreateExpr( 0), _memCopyExpr( 0)
		{}
	virtual void generate( 
			const std::string& hostFileName,
			const std::string& copyFileName,
			const std::string& meta, 
			const std::string& paradigm,
			const std::string& root,
			const std::string& dtd
		)
	{
		generateOpen( hostFileName, copyFileName, meta, paradigm, root, dtd, _udmProjectName, _dnsVecName, "Udm::CHANGES_LOST_DEFAULT");
		generateClose();
	}
	virtual void generateOpen(
		const std::string& hostFileName,
		const std::string& copyFileName,
		const std::string& meta,
		const std::string& paradigm,
		const std::string& root,
		const std::string& dtd,
		const std::string& udmPrj,
		const std::string& dnsvecname,
		const std::string& mode
		) = 0;
	virtual void generateClose() = 0;
	virtual void getOpenCode( PEComposite& addTo) const = 0;
	virtual void getCloseCode( PEComposite& addTo) const = 0;
	virtual void getDnCode( PEComposite& addTo) const = 0;
	virtual const std::string& getDN() const = 0;

public:
	static std::string _udmProjectName;
	static std::string _dnsVecName;

protected:
	//
	void generateCreateMemDN( const std::string& hostFileName, const std::string& meta, const std::string& paradigm, const std::string& root, const std::string& dtd);
	void generateCopyMemDN( const std::string& lhs, const std::string& rhs);
	//
	void addCreateMemCode( PEComposite& addTo) const;
	void addCopyMemCode( PEComposite& addTo) const;
	//
	const std::string& getMemFileName() const 
	{ 
		return _memDNDef->_variable;
	}
private:
	Definition* _memDNDef;
	Expression* _memDNCreateExpr;
	Expression* _memCopyExpr;
};

// DNCodePrinter is replicated (not a virtual base class), but it is abstract, so it is ok.
class ReadCopyToMemDNCodePrinter : public CopyToMemDNCodePrinter, protected ReadDNProjectCodePrinter
{
public:
	virtual void generateOpen(
		const std::string& hostFileName,
		const std::string& copyFileName,	// unused
		const std::string& meta,
		const std::string& paradigm,
		const std::string& root,
		const std::string& dtd,
		const std::string& udmPrj,
		const std::string& dnsvecname,
		const std::string& mode
		) 
	{
		//
		ReadDNProjectCodePrinter::generateOpenDN( hostFileName, meta, paradigm, dtd, mode);
		//
		generateCreateMemDN( hostFileName, meta, paradigm, root, dtd);
		generateCopyMemDN( getMemFileName(), ReadDNProjectCodePrinter::getSdnFileName());
		//
		std::string dnSpecifier= "\""+ PE::createUniqueName( meta)+ ".mem\"";
		ReadDNProjectCodePrinter::generateSDNS( getMemFileName(), dnSpecifier, meta, dnsvecname);
		ReadDNProjectCodePrinter::generateDNFromProject( dnSpecifier, meta, udmPrj);
	}
	virtual void generateClose()
	{
		ReadDNProjectCodePrinter::generateClose();
	}
	virtual void getOpenCode( PEComposite& addTo) const
	{
		ReadDNProjectCodePrinter::addOpenDNCode( addTo);
		addCreateMemCode( addTo);
		addCopyMemCode( addTo);
		ReadDNProjectCodePrinter::addSDNSCode( addTo);
	}
	virtual void getCloseCode( PEComposite& addTo) const
	{
		ReadDNProjectCodePrinter::getCloseCode( addTo);
	}
	virtual void getDnCode( PEComposite& addTo) const
	{	
		ReadDNProjectCodePrinter::getDnCode( addTo);
	}
	virtual const std::string& getDN() const
	{
		return ReadDNProjectCodePrinter::getDN();
	}
protected:

private:
};

class WriteCopyToMemDNCodePrinter : public CopyToMemDNCodePrinter, private WriteDNProjectCodePrinter
{
	virtual void generateOpen(
		const std::string& hostFileName,
		const std::string& copyFileName,	// unused
		const std::string& meta,
		const std::string& paradigm,
		const std::string& root,
		const std::string& dtd,
		const std::string& udmPrj,
		const std::string& dnsvecname,
		const std::string& mode
		) 
	{
		//
		generateCreateMemDN( hostFileName, meta, paradigm, root, dtd);
		//
		std::string dnSpecifier= "\""+ PE::createUniqueName( meta)+ ".mem\"";
		WriteDNProjectCodePrinter::generateSDNS( getMemFileName(), dnSpecifier, meta, dnsvecname);
		WriteDNProjectCodePrinter::generateDNFromProject( dnSpecifier, meta, udmPrj);
		//
		WriteDNProjectCodePrinter::generateOpenDN( hostFileName, meta, paradigm, root, dtd, mode);
	}
	virtual void generateClose()
	{
		generateCopyMemDN( WriteDNProjectCodePrinter::getSdnFileName(), getMemFileName());
		WriteDNProjectCodePrinter::generateClose();
	}
	virtual void getOpenCode( PEComposite& addTo) const
	{
		addCreateMemCode( addTo);
		WriteDNProjectCodePrinter::addSDNSCode( addTo);
	}
	virtual void getCloseCode( PEComposite& addTo) const
	{
		WriteDNProjectCodePrinter::addOpenDNCode( addTo);
		addCopyMemCode( addTo);
		WriteDNProjectCodePrinter::getCloseCode( addTo);
	}
	virtual void getDnCode( PEComposite& addTo) const
	{	
		WriteDNProjectCodePrinter::getDnCode( addTo);
	}
	virtual const std::string& getDN() const
	{
		return WriteDNProjectCodePrinter::getDN();
	}
};

class ReadWriteCopyToMemDNCodePrinter : public ReadCopyToMemDNCodePrinter
{
public:
	ReadWriteCopyToMemDNCodePrinter() : ReadCopyToMemDNCodePrinter(), _copyBackExpr( 0)
		{}
	virtual void generateClose()
	{
		generateCopyBackDN();
		ReadDNProjectCodePrinter::generateCloseDN( true);
	}
	virtual void getCloseCode( PEComposite& addTo) const
	{
		addCopyBackDN( addTo);
		ReadDNProjectCodePrinter::getCloseCode( addTo);
	}

protected:
	void generateCopyBackDN();
	void addCopyBackDN( PEComposite& addTo) const
	{
		if ( _copyBackExpr) addTo.add( *_copyBackExpr);
	}

private:
	Expression* _copyBackExpr;
};


class WriteCopyCopyToMemDNCodePrinter : public ReadWriteCopyToMemDNCodePrinter
{
public:
	WriteCopyCopyToMemDNCodePrinter() : ReadWriteCopyToMemDNCodePrinter(), _saveAsExpr( 0)
		{}
	virtual void generateOpen(
		const std::string& hostFileName,
		const std::string& copyFileName,	
		const std::string& meta,
		const std::string& paradigm,
		const std::string& root,
		const std::string& dtd,
		const std::string& udmPrj,
		const std::string& dnsvecname,
		const std::string& mode
		) 
	{
		ReadCopyToMemDNCodePrinter::generateOpen( hostFileName, copyFileName, meta, paradigm, root, dtd, udmPrj, dnsvecname, mode);
		_copyName= copyFileName;
	}
	virtual void generateClose()
	{
		generateCopyBackDN();
		generateSaveAsDN();
		ReadDNProjectCodePrinter::generateClose();
	}
	virtual void getCloseCode( PEComposite& addTo) const
	{
		addCopyBackDN( addTo);
		addSaveAsDN( addTo);
		ReadDNProjectCodePrinter::getCloseCode( addTo);
	}

protected:
	void generateSaveAsDN();
	void addSaveAsDN( PEComposite& addTo) const
	{
		if ( _saveAsExpr) addTo.add( *_saveAsExpr);
	}

private:
	std::string _copyName;
	Expression* _saveAsExpr;
};

#endif //COPYTOMEMDNCODEPRINTER_H