#include "SmartDNCodePrinter.h"
#include "../CodeGenerator/PEStringTable.h"
#include "GReATUtils.h"


SmartDNCodePrinter::SmartDNCodePrinter() : 
	_hostDef( 0),
	_hostOpenExpr( 0),
	_memDef( 0),
	_memCreateExpr( 0),
	_copyDNExpr( 0),
	_copyBackDNExpr( 0),
	_hostClose( 0),
	_memClose( 0)
{
}

void SmartDNCodePrinter::getOpenCode( PEComposite& addTo) const
{
	if ( _hostDef) addTo.add( *_hostDef);
	if ( _hostOpenExpr) addTo.add( *_hostOpenExpr);
	if ( _memDef) addTo.add( *_memDef);
	if ( _memCreateExpr) addTo.add( *_memCreateExpr);
	if ( _copyDNExpr) addTo.add( *_copyDNExpr);
}

void SmartDNCodePrinter::getCloseCode( PEComposite& addTo) const
{
	if ( _copyBackDNExpr) addTo.add( *_copyBackDNExpr);
	if ( _hostClose) addTo.add( *_hostClose);
	if ( _memClose) addTo.add( *_memClose);
}

void SmartDNCodePrinter::getDnCode( PEComposite& addTo) const
{
}

void SmartDNCodePrinter::createOpenCode( const std::string& fname, const std::string& meta, const std::string& paradigm, const std::string& root, const std::string& dtd)
{
	//
	_hostDef= constructObj( _hostDef);
	_hostDef->_type= "Udm::SmartDataNetwork";
	_hostDef->_variable= PE::createUniqueName( "snd"+ meta);
	_hostDef->addArg( meta+ RES+ "diagram");
	//
	_hostOpenExpr= constructObj( _hostOpenExpr);
	std::string metalocator= "UseXSD()("+ SEP+ fname+ ") ? "+ dtd+ " : \"" + paradigm + "\"";
	_hostOpenExpr->_expression= _hostDef->_variable+ ".OpenExisting("+ SEP+ fname+ ","+ SEP+
		metalocator+ ","+ SEP+  "Udm::CHANGES_LOST_DEFAULT)";
	//
	createOpenCodeWriteOnly( meta, root);
	//
	_copyDNExpr= constructObj( _copyDNExpr);
	_copyDNExpr->_expression= _memDef->_variable + EQ+ _hostDef->_variable;
}

void SmartDNCodePrinter::createOpenCodeWriteOnly( const std::string& meta, const std::string& root)
{
	//
	_memDef= constructObj( _memDef);
	_memDef->_type= "Udm::SmartDataNetwork";
	_memDef->_variable= PE::createUniqueName( "sndMem"+ meta);
	_memDef->addArg( meta+ RES+ "diagram");
	//
	_memCreateExpr= constructObj( _memCreateExpr);
	_memCreateExpr->_expression= _memDef->_variable+ ".CreateNew("+ SEP+ "\""+ PE::createUniqueName( "dummy"+ meta)+ ".mem\","+ SEP+
		"\""+ meta+ "\","+ SEP+ meta+ RES+ root+ RES+ "meta," + SEP+ "Udm::CHANGES_LOST_DEFAULT)";
}

void SmartDNCodePrinter::createCopyMemToHostDN()
{
	createCopyMemToDN( _hostDef->_variable);
}

void SmartDNCodePrinter::createCopyMemToDN( const std::string& dn)
{
	_copyBackDNExpr= constructObj( _copyBackDNExpr);
	_copyBackDNExpr->_expression= dn+ EQ+ _memDef->_variable;
}

void SmartDNCodePrinter::createCloseDNs( bool closeHostWithUpdate, bool closeMemWithUpdate)
{
	createCloseHost( closeHostWithUpdate);
	createCloseMem( closeMemWithUpdate);
}

void SmartDNCodePrinter::createCloseHost( bool closeHostWithUpdate)
{
	const std::string& CLOSE_WITH_UPDATE= ".CloseWithUpdate()";
	const std::string& CLOSE_NO_UPDATE= ".CloseNoUpdate()";
	_hostClose= constructObj( _hostClose);
	_hostClose->_expression= _hostDef->_variable + ( closeHostWithUpdate ? CLOSE_WITH_UPDATE : CLOSE_NO_UPDATE);
}

void SmartDNCodePrinter::createCloseMem(  bool closeMemWithUpdate)
{
	const std::string& CLOSE_WITH_UPDATE= ".CloseWithUpdate()";
	const std::string& CLOSE_NO_UPDATE= ".CloseNoUpdate()";
	_memClose= constructObj( _memClose);
	_memClose->_expression= _memDef->_variable + ( closeMemWithUpdate ? CLOSE_WITH_UPDATE : CLOSE_NO_UPDATE);
}

///////////////////////////////////////////////////////////////////////////////
SmartDNWriteOnlyCodePrinter::SmartDNWriteOnlyCodePrinter() :
	_newHostDef( 0),
	_createNewHostExpr( 0),
	_closeNewHostExpr( 0)
{
}

void SmartDNWriteOnlyCodePrinter::createNewDN( const std::string& fname, const std::string& meta, const std::string& paradigm, const std::string& root, const std::string& dtd)
{
	//
	_newHostDef= constructObj( _newHostDef);
	_newHostDef->_type= "Udm::SmartDataNetwork";
	_newHostDef->_variable= PE::createUniqueName( getNewHostPrefix()+ meta);
	_newHostDef->addArg( meta+ RES+ "diagram");
	//
	_createNewHostExpr= constructObj( _createNewHostExpr);
	std::string metalocator= "UseXSD()("+ SEP+ fname+ ") ? "+ dtd+ " : \"" + paradigm + "\"";
	_createNewHostExpr->_expression= _newHostDef->_variable+ ".CreateNew("+ SEP+ fname+ ","+ SEP+
		metalocator+ ","+ SEP+ meta+ RES+ root+ RES+ "meta," + SEP+ "Udm::CHANGES_LOST_DEFAULT)";
}

void SmartDNWriteOnlyCodePrinter::createCopyMemToNewDN()
{
	createCopyMemToDN( _newHostDef->_variable);
}

void SmartDNWriteOnlyCodePrinter::createCloseNewDN()
{
	_closeNewHostExpr= constructObj( _closeNewHostExpr);
	_closeNewHostExpr->_expression= _newHostDef->_variable+ ".CloseWithUpdate()";
}

