/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
*/


// UdmApp.cpp: implementation of the CUdmApp class.
// This file was automatically generated as UdmApp.cpp
// by UDM Interpreter Wizard on Monday, May 13, 2002 13:45:42

// Tihamr Levendovszky 05-13-02

#include "stdafx.h"
#include <afxdlgs.h> // For CFileDialog
#include "resource.h"

//#include <iostream.h>
#include <atlbase.h>
#include <comdef.h>

#include <time.h>
#include "UdmStatic.h"
#include "UdmDom.h"
#include "UdmGME.h"
#include "UmlExt.h"
#include "UdmUtil.h"

#include "UdmApp.h"
#include "UdmConfig.h"

#include "resource.h"

#include <direct.h>
#include <process.h>

//#include "GReATConfig.h"
#include "GReATConfigMeta.h"
#include "GReATUtils.h"

extern void dummy(void); // Dummy function for UDM meta initialization


char* GetRelativeFilename(const char *currentDirectory,const char *absoluteFilename);
bool isSameDrive(const char *currentDirectory,const char *absoluteFilename);

string convertPath(const string &currentPath, const string &absoluteFilename)
{
	string path(absoluteFilename);
	if(!absoluteFilename.empty() && 
		isSameDrive(currentPath.c_str(), absoluteFilename.c_str()))
		path = (string)(GetRelativeFilename(currentPath.c_str(),absoluteFilename.c_str()));

	return path;
}

static string currPath = "";
/*********************************************************************************/
/* Initialization function. The framework calls it before preparing the backend. */
/* Initialize here the settings in the config static object.					 */
/* Return 0 if successful.														 */
/*********************************************************************************/
int CUdmApp::Initialize()
{
	// TODO: Your initialization code comes here...
	return 0;
}

/* 
Remarks to CUdmApp::UdmMain(...):
0.	The p_backend points to an already open backend, and the framework 
	closes it automatically. DO NOT OPEN OR CLOSE IT!
	To commit changes use p_backend->CommitEditSequence().
	To abort changes use p_backend->AbortEditSequence().
	To save changes to a different file use p_backend->SaveAs() or p_backend->CloseAs().

1.	Focus is the currently open model.

2.	The possible values for param (from GME DecoratorLib.h component_startmode_enum):
	GME_MAIN_START			=   0,
	GME_BROWSER_START		=   1,
	GME_CONTEXT_START		=   2,
	GME_EMBEDDED_START		=   3,
	GME_MENU_START			=  16,
	GME_BGCONTEXT_START		=  18,
	GME_ICON_START			=  32,
	METAMODEL_CHECK_SYNTAX	= 101

 3. The framework catches all the exceptions and reports the error in a message box,
	clean up and close the transactions aborting the changes. You can override this 
	behavior by catching udm_exception. Use udm_exception::what() to form an error 
	message.
*/

/***********************************************/
/* Main entry point for Udm-based Interpreter  */
/***********************************************/
//CGREngineDlgDlg* CUdmApp::m_pDialog = 0;

CString CUdmApp::mgaPath = "";

void CUdmApp::UdmMain(
					 Udm::DataNetwork* p_backend,		// Backend pointer(already open!)
					 Udm::Object focusObject,			// Focus object
					 set<Udm::Object> selectedObjects,	// Selected objects
					 long param)						// Parameters
{	
		UMLModelTransformer::RootFolder rf = UMLModelTransformer::RootFolder::Cast(p_backend->GetRootObject());
		const set<UMLModelTransformer::Configurations> &cfgs = rf.Configurations_kind_children();
		if(cfgs.empty()) throw udm_exception("No Configuration File found.");

		const UMLModelTransformer::Configurations &cfg = *(cfgs.begin());		
		const set<UMLModelTransformer::Configuration> &cfgfiles = cfg.Configuration_kind_children();
		if(cfgs.empty()) throw udm_exception("No Configurtion File found.");

		UMLModelTransformer::Configuration cfgfile = *(cfgfiles.begin());

//Get the full path from the greate_cfg
    const std::string& greate_cfg = getFullPath((string)mgaPath, (string)cfgfile.ConfigFile());


   const char *args[3];

   /* Set up parameters to be sent: */
   long BUFSIZE = 500;
   TCHAR tchBuf[500]; 
   std::string grdPath;
   if (GetEnvironmentVariable("GREAT_PATH",tchBuf, BUFSIZE))
   {
    grdPath = CString(tchBuf);
    grdPath +=  "\\bin\\grd.exe";
   }
   else
    throw udm_exception("Variable GREAT_PATH is not defined.");

//    std::string arg0 =   grdPath;
//    std::string arg1 =   "\" + greate_cfg + "\""  ;
//    args[0] = arg0.c_str();
//    args[1] = arg1.c_str();
//    args[2] = NULL;
//     
//     _spawnvp( _P_NOWAIT , args[0], args );



   std::string makeCall =  "\"" + grdPath + "\"";
    makeCall +=  " \"" + greate_cfg + "\""  ;

    CString mcall(makeCall.c_str());


    char *tmpstr = new char[mcall.GetLength()+1];
    strcpy(tmpstr, mcall);


    STARTUPINFO si;
    PROCESS_INFORMATION pi;

    ZeroMemory( &si, sizeof(si) );
    si.cb = sizeof(si);
    ZeroMemory( &pi, sizeof(pi) );

    // Start the child process. 
    if( !CreateProcess( NULL, // No module name (use command line). 
        tmpstr , // Command line. 
        NULL,             // Process handle not inheritable. 
        NULL,             // Thread handle not inheritable. 
        FALSE,            // Set handle inheritance to FALSE. 
        0,                // No creation flags. 
        NULL,             // Use parent's environment block. 
        NULL,             // Use parent's starting directory. 
        &si,              // Pointer to STARTUPINFO structure.
        &pi )             // Pointer to PROCESS_INFORMATION structure.
    ) 
    {
        throw udm_exception(" grd createProcess failed." );
    }

    // Wait until child process exits.
    WaitForSingleObject( pi.hProcess, INFINITE );

    // Close process and thread handles. 
    CloseHandle( pi.hProcess );
    CloseHandle( pi.hThread );
    delete []tmpstr;
}


#ifdef _DEBUG
/*****************************************************/
/* Debug time helper function. If the object has an  */
/* attribute called "name", this function retreives  */
/* it to help you to find it in the model during the */
/* application development.	Usualy every GME Object	 */
/* has a "name" attribute. If an object hapens not	 */
/* to have it,function retreives <no name specified>.*/
/*****************************************************/
string CUdmApp::ExtractName(Udm::Object ob)
{
	Uml::Class cls= ob.type();				
	set<Uml::Attribute> attrs=cls.attributes();		
	
	// Adding parent attributes
	set<Uml::Attribute> aattrs=Uml::AncestorAttributes(cls);
	attrs.insert(aattrs.begin(),aattrs.end());

	for(set<Uml::Attribute>::iterator ai = attrs.begin();ai != attrs.end(); ai++) 
	{
		if(string(ai->type())=="String")
		{
			string str=ai->name();
			if(str=="name")
			{
				string value=ob.getStringAttr(*ai);
				if(value.empty())value="<empty string>";
				return value;
			}			
		}				
	}	
	return string("<no name specified>");
}
#endif