#if !defined(IGRDebuggerView_h)
#define IGRDebuggerView_h

#include "../GRDebugInterface/GRDebugTypes.h"
#include <sstream>

namespace GRDI
{
class IGRDebuggerView
{
public:
  enum {CURR = 1, BREAK = 2, SET_BREAK = 4, HIT_BREAK = 8, PAUSE = 16} LocationMark;
  typedef struct
  {
    Location m_loc;
    FunctionDescr m_descr;
    int   m_mark;
  } FuncLocationMarkDescr;

  typedef std::vector<FuncLocationMarkDescr> FuncLocationMarkDescrVec;

public:
    virtual ~IGRDebuggerView() {}

    virtual void print(const std::string&) = 0;

    virtual void buildFunctionSourceImpl( const Location& loc,
                                          const FunctionDescr& descr, 
                                          std::stringstream& str) = 0;

    virtual void buildSourceImplBegin(std::stringstream& str) = 0;
    virtual void buildSourceImplEnd(std::stringstream& str) = 0;
    virtual void markFuncLocationImpl(  std::stringstream& str,
                                        FuncLocationMarkDescrVec& locs) = 0;
    virtual void printSource() = 0;

    virtual void printContextImpl(const ContextDescr& contextDescr, 
                                  std::ostream& str,
                                  bool exception) = 0;
    virtual void printContextStackImplBegin(std::ostream& str, bool exception) = 0;
    virtual void printContextStackImplEnd(std::ostream& str, bool exception) = 0;

    virtual void stopped() = 0;
    virtual void exception() = 0;
    virtual void finished() = 0;
    virtual void loaded() = 0;
    virtual void saved() = 0;
    virtual void closed() = 0;  

    virtual void hitBreakpoint() = 0;
    virtual void step() = 0;
    virtual void progressReport(double) = 0;
    virtual void printCurrentProgress(double) = 0;
};

};//namespace GRDI
#endif //IGRDebuggerView_h