// GRDebuggerGUIDoc.cpp : implementation of the CGRDebuggerGUIDoc class
//

#include "stdafx.h"

#include "Messages.h"

#include "GRDebuggerGUI.h"

#include "GRDebuggerGUIDoc.h"
#include "MainFrm.h"


#include "GRDebuggerGUIView.h"

#include "SourceTxtView.h"
#include "StackView.h"

// #include "../Log/Log.h"
// extern Log lout;


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGRDebuggerGUIDoc

IMPLEMENT_DYNCREATE(CGRDebuggerGUIDoc, CDocument)

BEGIN_MESSAGE_MAP(CGRDebuggerGUIDoc, CDocument)
	//{{AFX_MSG_MAP(CGRDebuggerGUIDoc)
	ON_COMMAND(ID_STEP, OnStep)
	ON_UPDATE_COMMAND_UI(ID_STEP, OnUpdateStep)
	ON_COMMAND(ID_STEPOVER, OnStepover)
	ON_UPDATE_COMMAND_UI(ID_STEPOVER, OnUpdateStepover)
	ON_COMMAND(ID_START, OnStart)
	ON_UPDATE_COMMAND_UI(ID_START, OnUpdateStart)
	ON_COMMAND(ID_STOP, OnStop)
	ON_UPDATE_COMMAND_UI(ID_STOP, OnUpdateStop)
	ON_COMMAND(ID_STEPOUT, OnStepout)
	ON_UPDATE_COMMAND_UI(ID_STEPOUT, OnUpdateStepout)
	ON_COMMAND(ID_PAUSE, OnPause)
	ON_UPDATE_COMMAND_UI(ID_PAUSE, OnUpdatePause)
	ON_COMMAND(ID_CONT, OnCont)
	ON_UPDATE_COMMAND_UI(ID_CONT, OnUpdateCont)
  ON_COMMAND_RANGE( ID_POPUP_DynCmd, ID_POPUP_DynCmd_Max	, OnVariablePopUp )
  ON_COMMAND_RANGE( ID_POPUP_GOTO_ID, ID_POPUP_GOTO_ID, OnGotoLocationPopUp )
	ON_COMMAND(ID_GR_CLOSE, OnGrClose)
	ON_UPDATE_COMMAND_UI(ID_GR_CLOSE, OnUpdateGrClose)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
	ON_UPDATE_COMMAND_UI(ID_FILE_OPEN, OnUpdateFileOpen)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGRDebuggerGUIDoc construction/destruction

CGRDebuggerGUIDoc::CGRDebuggerGUIDoc() :m_prevCtxId(0),
m_ctrlEnabled(false),
m_pauseEnabled(false),
m_stopEnabled(false),
m_loading(false),
m_closing(false),
m_running(false),
m_pMainFrame(0),
m_logView(0),
m_srcView(0),
m_stackView(0),
m_stackViewChanged(true)
{
	// TODO: add one-time construction code here
initDebugger();
}

CGRDebuggerGUIDoc::~CGRDebuggerGUIDoc()
{

}

BOOL CGRDebuggerGUIDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

   POSITION pos = GetFirstViewPosition();
   m_srcView = (SourceTxtView*) GetNextView(pos);
 
   m_pMainFrame = (CMainFrame*) m_srcView->GetParentFrame( );
   m_logView = m_pMainFrame->m_logViewFrame->m_view;
   m_stackView = (StackView*) (m_pMainFrame->m_stackViewFrame->m_view);

   AddView(m_logView);
   AddView(m_stackView);


   startDebugger();
   m_pMainFrame->StatusBarMessage("Ready"); 

	return TRUE;
}

//=========================================
void CGRDebuggerGUIDoc::initDebugger()
{
  if (m_launcher.init(this))
    m_launcher.run();
  else
  {
    AfxMessageBox("Launching Exception!");
    exit(-1);
  }
}
//=====================================

void CGRDebuggerGUIDoc::startDebugger()
{
  m_debuggerCtrl->startCtrl();
}
//=====================================
void CGRDebuggerGUIDoc::closeDebugger()
{
 m_debuggerCtrl->stepCtrl("quit");
 m_debuggerCtrl->stopCtrl();
 m_launcher.stop();
 m_launcher.close();
}
//=====================================
void CGRDebuggerGUIDoc::setIGRDebuggerCtrl(GRDI::IGRDebuggerCtrl* debuggerCtrl)
{
  m_debuggerCtrl = debuggerCtrl;
}

//=====================================
GRDI::IGRDebuggerView* CGRDebuggerGUIDoc::getIGRDebuggerView()
{
  return static_cast<GRDI::IGRDebuggerView*>(this);
}
//=====================================
void CGRDebuggerGUIDoc::update()
{
  UpdateAllViews(NULL);
}
//=====================================
void CGRDebuggerGUIDoc::debuggerCtrl(const std::string& cmd)
{
  m_debuggerCmd = cmd;
  m_srcView->PostMessage(DEBUGGER_EXECUTE_CMD, 0, 0);
}
//=====================================
//======IGRDebuggerView

void CGRDebuggerGUIDoc::print(const std::string& str)
{
   Lock lk(m_mutexPrint); 
 
    CString a(str.c_str());
    a.Replace("\n", "\r\n");
 
    m_txt += a;
 
    m_logView->PostMessage(DEBUGGER_LOG_UPDATE, 0, 0);
}

//=========================================
void CGRDebuggerGUIDoc::buildFunctionSourceImpl( const GRDI::Location& loc,
                              const GRDI::FunctionDescr& descr, 
                              std::stringstream& str)
{

   std::stringstream name;
   name << loc.m_locID << " " << *(loc.called());
 
  
    if (descr.m_varTable.size())
    {
      name << " ( ";
      const GRDI::VariableValueMap& vars = descr.m_varTable;
      GRDI::VariableValueMap::const_iterator itVars = vars.begin();
      for(;itVars != vars.end(); ++itVars)
      {
        if (itVars != vars.begin()) 
           name << ", ";

        name << *(itVars->first);
      }
      name << " )";
    }


   SourceTxtView::SrcItemType type = SourceTxtView::CALL;
   if (descr.m_ref)
   {
     type = (SourceTxtView::SrcItemType)(type | SourceTxtView::REFERENCE);
     name << "->" << *(descr.m_ref);
   }
   HTREEITEM treeItem = m_srcView->addTreeItem(loc.m_parID, loc.m_locID, name.str(), type);
   m_srcView->addTreeItemVars(treeItem, descr.m_varTable); 

   if (descr.m_ref == GRDI::IFunctionPtr())
   {
     const GRDI::LocationIDVec&  locations = descr.m_callLocs;
     const GRDI::FunctionTable&  calls = descr.m_calls;    
 
     std::stringstream begin;
     begin << locations.front() << " begin";
     type = (SourceTxtView::SrcItemType)(type | SourceTxtView::BOUNDARY);
     m_srcView->addTreeItem(loc.m_locID, locations.front(), begin.str(), type );
 
     for(GRDI::FunctionTable::size_type i = 0; i < calls.size(); ++i)
     {
       const GRDI::LocationID& locID = locations[i + 1];
       std::stringstream name;
       name << " " << *(descr.m_calls[i]);
       m_srcView->addTreeItem(loc.m_locID, locID, name.str());
     }
     std::stringstream end;
     end << locations.back() << " end";
     m_srcView->addTreeItem(loc.m_locID, locations.back(), end.str(), type );
   }
   else
     m_srcView->addRefMap(loc.m_locID, loc.m_refID);
}
//=========================================
void CGRDebuggerGUIDoc::markFuncLocationImpl(  std::stringstream& str,
                                             GRDI::IGRDebuggerView::FuncLocationMarkDescrVec& locs)
{  
  m_markLocs = locs;
  m_sourceViewChanged = true;
  m_srcView->PostMessage(DEBUGGER_VIEW_UPDATE, 0, 0);
}
//=========================================
void CGRDebuggerGUIDoc::enableCtrl()
{
  m_ctrlEnabled = true;
  m_pauseEnabled = false;
  m_stopEnabled = true;
}

//=========================================
void CGRDebuggerGUIDoc::disableCtrl()
{
  m_ctrlEnabled = false;
  m_pauseEnabled = false;
  m_stopEnabled = false;
}

//=========================================
void CGRDebuggerGUIDoc::stackView()
{ 
  if (m_stackViewChanged)
  {
    m_srcView->PostMessage(DEBUGGER_VIEW_UPDATE, 0, 0);
  }
}
//=========================================
void CGRDebuggerGUIDoc::executeCmd()
{ 
  m_debuggerCtrl->stepCtrl(m_debuggerCmd);
  m_stackViewChanged = true;
}
//=========================================
void CGRDebuggerGUIDoc::handleException()
{
  disableCtrl();
  buildSourceImplBegin(std::stringstream());
  SetPathName("No program loaded", FALSE);
  m_loading = false;
  m_pMainFrame->StatusBarMessage("Ready"); 
}
//=========================================
void CGRDebuggerGUIDoc::buildSourceImplBegin(std::stringstream& str)
{
  m_srcView->clear(); // FIXME: crash on `grd.exe NONEXISTANT` due to m_srcView being destructed
  m_stackView->showEmptyText();
  
}

//=========================================
void CGRDebuggerGUIDoc::buildSourceImplEnd(std::stringstream& str)
{
  m_srcView->expandRoot();
  


  enableCtrl();
}
//=========================================
void CGRDebuggerGUIDoc::printSource()
{
//  m_srcView->expandRoot();
}
//=========================================
void CGRDebuggerGUIDoc::printContextImpl(const GRDI::ContextDescr& contextDescr, 
                                  std::ostream& str,
                                  bool exception)
{
 if (!exception)
 {
  m_stackView->addCtx(contextDescr);
 }
}
//=========================================
void CGRDebuggerGUIDoc::printContextStackImplBegin(std::ostream& str, 
                                              bool exception)
{
  m_stackView->clear();
}

//=========================================
void CGRDebuggerGUIDoc::printContextStackImplEnd(std::ostream& str, 
                                              bool exception)
{
 if (!exception)
 {
  m_stackView->expandStack();
 }
 enableCtrl();
}
//=========================================
void CGRDebuggerGUIDoc::stopped()
{

  m_stackView->showEmptyText();
  enableCtrl();
  progressReport(0);
  m_pMainFrame->StatusBarMessage("Ready"); 
  m_running = false;

}
//=========================================
void CGRDebuggerGUIDoc::exception()
{
  m_srcView->PostMessage(DEBUGGER_EXECEPTION, 0, 0);
  AfxMessageBox("Exception! See details in the log window.");
  m_running = false;
  progressReport(0);
}

//=========================================
void CGRDebuggerGUIDoc::finished()
{

  m_stackView->showEmptyText();
  m_srcView->hideCursor();
  enableCtrl();
  progressReport(0);
  m_pMainFrame->StatusBarMessage("Ready"); 
  m_running = false;

}

//=========================================
void CGRDebuggerGUIDoc::loaded()
{
  enableCtrl();
  
  m_pMainFrame->StatusBarMessage("Ready"); 
  m_loading = false;
  m_running = false;
}
//=========================================
void CGRDebuggerGUIDoc::saved()
{
  enableCtrl();

  m_pMainFrame->StatusBarMessage("Ready"); 
}

//=========================================
void CGRDebuggerGUIDoc::closed()
{
  m_closing = false;
  m_running = false;
  m_pMainFrame->StatusBarMessage("Ready"); 
  progressReport(0);

}

//=========================================
void CGRDebuggerGUIDoc::hitBreakpoint()
{

  stackView();
  progressReport(0);
  m_pMainFrame->StatusBarMessage("Ready"); 
}

//=========================================
void CGRDebuggerGUIDoc::step()
{
  stackView();
  progressReport(0);
  m_pMainFrame->StatusBarMessage("Ready"); 
}
//=========================================
void CGRDebuggerGUIDoc::progressReport(double prg)
{
  m_pMainFrame->SetProgress(prg);
}
//=========================================
void CGRDebuggerGUIDoc::printCurrentProgress(double prg)
{

  progressReport(prg);
}

/////////////////////////////////////////////////////////////////////////////
// CGRDebuggerGUIDoc serialization

void CGRDebuggerGUIDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CGRDebuggerGUIDoc diagnostics

#ifdef _DEBUG
void CGRDebuggerGUIDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CGRDebuggerGUIDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////
// CGRDebuggerGUIDoc commands

void CGRDebuggerGUIDoc::viewUpdate()
{
   
  if (m_sourceViewChanged)
  {
    m_srcView->markItem(m_markLocs);
    m_sourceViewChanged = false;
  }
  debuggerCtrl("where");
  m_stackViewChanged = false;
}
//=============================================
void CGRDebuggerGUIDoc::OnCloseDocument() 
{
  OnGrClose();
  closeDebugger();	  
	CDocument::OnCloseDocument();
}

//=============================================
void CGRDebuggerGUIDoc::OnStep() 
{
  m_pMainFrame->StatusBarMessage("Stepping..."); 
  m_ctrlEnabled = false;  
  debuggerCtrl("step");

  m_running = true;
  m_pauseEnabled = true;
}
//=============================================

void CGRDebuggerGUIDoc::OnUpdateStep(CCmdUI* pCmdUI) 
{
  if (m_ctrlEnabled)
  	pCmdUI->Enable( TRUE );
  else
    pCmdUI->Enable( FALSE );

}

void CGRDebuggerGUIDoc::OnStepover() 
{
  m_pMainFrame->StatusBarMessage("Stepping Over..."); 
  m_ctrlEnabled = false;
	debuggerCtrl("next");
  m_running = true;
  m_pauseEnabled = true;
}

void CGRDebuggerGUIDoc::OnUpdateStepover(CCmdUI* pCmdUI) 
{
  if (m_ctrlEnabled)
	  pCmdUI->Enable( TRUE );
  else
    pCmdUI->Enable( FALSE );
	
}

void CGRDebuggerGUIDoc::OnStart() 
{
  m_pMainFrame->StatusBarMessage("Executing..."); 
  m_stackViewChanged = true;
  disableCtrl();
  m_txt.Empty();

  debuggerCtrl("start");
  m_srcView->hideCursor();
  m_running = true;
  m_stopEnabled = true;

}

void CGRDebuggerGUIDoc::OnUpdateStart(CCmdUI* pCmdUI) 
{
  if (!m_running && m_ctrlEnabled)
    pCmdUI->Enable( TRUE );
  else
    pCmdUI->Enable( FALSE );

}

void CGRDebuggerGUIDoc::OnStop() 
{
  m_pMainFrame->StatusBarMessage("Stopping..."); 
	debuggerCtrl("stop");
  m_srcView->hideCursor();
  
}

void CGRDebuggerGUIDoc::OnUpdateStop(CCmdUI* pCmdUI) 
{
  if (m_stopEnabled)
    pCmdUI->Enable( TRUE );
  else
   pCmdUI->Enable( FALSE );
}

void CGRDebuggerGUIDoc::OnStepout() 
{
  m_pMainFrame->StatusBarMessage("Stepping out..."); 
	m_ctrlEnabled = false;  
  debuggerCtrl("step out");
  m_pauseEnabled = true;
}

void CGRDebuggerGUIDoc::OnUpdateStepout(CCmdUI* pCmdUI) 
{
  if (m_ctrlEnabled)
	  pCmdUI->Enable( TRUE );
  else
    pCmdUI->Enable( FALSE );
}


BOOL CGRDebuggerGUIDoc::SaveModified() 
{
	// TODO: Add your specialized code here and/or call the base class
	
//	return CDocument::SaveModified();
  	return TRUE;
}

BOOL CGRDebuggerGUIDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
  progressReport(0);
  m_loading = true;

  m_ctrlEnabled = false;  
  m_srcView->clear();
  m_pMainFrame->StatusBarMessage("Loading..."); 	
  std::string a("load ");
  
  a += std::string(lpszPathName);
 	debuggerCtrl(a);

	m_lpszPathName = lpszPathName;
	return TRUE;
}

BOOL CGRDebuggerGUIDoc::OnSaveDocument(LPCTSTR lpszPathName) 
{
  m_ctrlEnabled = false;  
  m_pMainFrame->StatusBarMessage("Saving..."); 
 	debuggerCtrl("save");
	return TRUE;
}

void CGRDebuggerGUIDoc::OnPause() 
{
  m_pMainFrame->StatusBarMessage("Pause..."); 
  m_ctrlEnabled = false;  
	debuggerCtrl("pause");
}

void CGRDebuggerGUIDoc::OnUpdatePause(CCmdUI* pCmdUI) 
{
  if (m_pauseEnabled)
	  pCmdUI->Enable( TRUE );
  else
    pCmdUI->Enable( FALSE );
	
}

void CGRDebuggerGUIDoc::OnCont() 
{
  m_ctrlEnabled = false;  
  m_pMainFrame->StatusBarMessage("Running..."); 
  m_srcView->hideCursor();
	debuggerCtrl("run");
  m_pauseEnabled = true;
}

void CGRDebuggerGUIDoc::OnUpdateCont(CCmdUI* pCmdUI) 
{
  if (m_ctrlEnabled)
	  pCmdUI->Enable( TRUE );
  else
    pCmdUI->Enable( FALSE );
}

void CGRDebuggerGUIDoc::OnVariablePopUp( UINT nID )
{
  GRDI::IVariablePtr var =  m_srcView->getVariable(nID);

  if (var != GRDI::IVariablePtr())
    m_stackView->showVar(var);
}

void CGRDebuggerGUIDoc::OnGotoLocationPopUp( UINT nID )
{

    m_srcView->selectLocation(m_stackView->m_showLocID);
}


void CGRDebuggerGUIDoc::OnGrClose() 
{
  m_closing = true;
  disableCtrl();
  progressReport(0);
  buildSourceImplBegin(std::stringstream());
  m_pMainFrame->StatusBarMessage("Closing..."); 
	debuggerCtrl("close");
  SetPathName("No program loaded", FALSE);

}

void CGRDebuggerGUIDoc::OnUpdateGrClose(CCmdUI* pCmdUI) 
{
  if (m_ctrlEnabled)
 	  pCmdUI->Enable( TRUE );
  else
    pCmdUI->Enable( FALSE );
}

void CGRDebuggerGUIDoc::OnUpdateFileSave(CCmdUI* pCmdUI) 
{
//  pCmdUI->Enable( FALSE ); // not implemented;
  if (m_ctrlEnabled)
 	  pCmdUI->Enable( TRUE );
  else
    pCmdUI->Enable( FALSE );
}

void CGRDebuggerGUIDoc::OnUpdateFileOpen(CCmdUI* pCmdUI) 
{
  if (!m_loading && !m_closing)
 	  pCmdUI->Enable( TRUE );
  else
    pCmdUI->Enable( FALSE );
	
}

// void CGRDebuggerGUIDoc::OnUpdateFileMruFile1(CCmdUI* pCmdUI) 
// {
// 
//   if (!m_loading && !m_closing)
//  	  pCmdUI->Enable( TRUE );
//   else
//     pCmdUI->Enable( FALSE );
// 
// }

