// SourceTxtView.cpp : implementation file
//

#include "stdafx.h"
#include "Messages.h"
#include "GRDebuggerGUI.h"
#include "SourceTxtView.h"
#include "GRDebuggerGUIDoc.h"
#include "../GReATDebugger/GReATDebuggerVars.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// SourceTxtView

IMPLEMENT_DYNCREATE(SourceTxtView, CTreeView)

SourceTxtView::SourceTxtView() : m_prevIt(0), m_log(false), m_imageList(0)
{
  bm_in = new CBitmap();
  bm_in->LoadBitmap(IDB_LOCALVARIABLESIN);

  bm_out = new CBitmap();
  bm_out->LoadBitmap(IDB_LOCALVARIABLESOUT);

  bm_inout = new CBitmap();
  bm_inout->LoadBitmap(IDB_LOCALVARIABLESINOUT);

}

SourceTxtView::~SourceTxtView()
{
 if (bm_in)
    delete bm_in, bm_in = 0;
 if (bm_out)
    delete bm_out, bm_out = 0;
 if (bm_inout)
    delete bm_inout, bm_inout = 0;

 if (m_imageList)
   delete m_imageList, m_imageList = 0;
}


BEGIN_MESSAGE_MAP(SourceTxtView, CTreeView)
	//{{AFX_MSG_MAP(SourceTxtView)
	ON_COMMAND(ID_TOGGLEBREAKPOINT, OnTogglebreakpoint)
	ON_UPDATE_COMMAND_UI(ID_TOGGLEBREAKPOINT, OnUpdateTogglebreakpoint)
	ON_COMMAND(ID_LOG, OnLog)
	ON_UPDATE_COMMAND_UI(ID_LOG, OnUpdateLog)
	ON_COMMAND(ID_SHOWALLREAKPOINTS, OnShowallreakpoints)
	ON_UPDATE_COMMAND_UI(ID_SHOWALLREAKPOINTS, OnUpdateShowallreakpoints)
	ON_COMMAND(ID_CLEARALL, OnClearall)
	ON_UPDATE_COMMAND_UI(ID_CLEARALL, OnUpdateClearall)
	ON_WM_LBUTTONDBLCLK()
  ON_MESSAGE( DEBUGGER_VIEW_UPDATE, OnViewUpdate)
  ON_MESSAGE( DEBUGGER_EXECUTE_CMD, OnExecuteCmd)
  ON_MESSAGE( DEBUGGER_EXECEPTION,  OnException)
	ON_WM_RBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



/////////////////////////////////////////////////////////////////////////////
// SourceTxtView diagnostics

#ifdef _DEBUG
void SourceTxtView::AssertValid() const
{
	CTreeView::AssertValid();
}

void SourceTxtView::Dump(CDumpContext& dc) const
{
	CTreeView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
//=====================================
LRESULT SourceTxtView::OnExecuteCmd(WPARAM, LPARAM)
{
  CGRDebuggerGUIDoc* pDoc = (CGRDebuggerGUIDoc* )GetDocument();
  pDoc->executeCmd();
  return 0;
}
//=====================================
LRESULT SourceTxtView::OnException(WPARAM, LPARAM)
{
  CGRDebuggerGUIDoc* pDoc = (CGRDebuggerGUIDoc* )GetDocument();
  pDoc->handleException();
  return 0;
}

//=====================================
LRESULT SourceTxtView::OnViewUpdate(WPARAM, LPARAM)
{
  CGRDebuggerGUIDoc* pDoc = (CGRDebuggerGUIDoc* )GetDocument();
  pDoc->viewUpdate();
  return 0;
}
//=====================================
HTREEITEM SourceTxtView::findTreeItem(const GRDI::LocationID& locID)
{

 LocIDItemMap::iterator itCh = m_idMap.find(locID);
 if (itCh == m_idMap.end())
   throw std::invalid_argument("invalid location");
 return itCh->second;
}

//=====================================
void  SourceTxtView::expandRoot()
{
  GetTreeCtrl().Expand(m_rootItem, TVE_EXPAND);
  setCallLocationImage(m_rootItem);
}

//=============================================
void SourceTxtView::setCallLocationImage(HTREEITEM it )
{
  SrcItemType  type =  (SrcItemType) GetTreeCtrl().GetItemData(it);
  int imNum = 0;
  if (type & CURRENT)
  {
    if (type  & BREAKPOINT)
      imNum  = 4;
    else
      imNum  = 2;
  }
  else
  if (type & BREAKPOINT)
  {
     imNum  = 3;
  }
  else
  if ((TVIS_EXPANDED   & GetTreeCtrl().GetItemState(it, TVIS_EXPANDED)) ||
      (type & BOUNDARY)) 
    imNum = 1;

   GetTreeCtrl().SetItemImage(it, imNum, imNum);
}

//=====================================
HTREEITEM SourceTxtView::addTreeItem(const GRDI::LocationID& parID,
                                         const GRDI::LocationID& childID,
                                         const std::string& name,
                                         int type)
{
   LocIDItemMap::const_iterator itCh = m_idMap.find(childID);

   if (itCh == m_idMap.end())
   {
     LocIDItemMap::const_iterator itPar = m_idMap.find(parID);
     if (itPar == m_idMap.end())
        m_idMap[childID] = GetTreeCtrl().InsertItem(name.c_str(), NULL);
     else
       m_idMap[childID] = GetTreeCtrl().InsertItem(name.c_str(), itPar->second);

   }
   else
     GetTreeCtrl().SetItemText(itCh->second, name.c_str());

   HTREEITEM it = m_idMap[childID];
   m_itemMap[it] = childID;

   GetTreeCtrl().SetItemData(it, type );
   setCallLocationImage(it);

   if (childID == 0)
     m_rootItem = it;

   
   return it;
}
//=============================================
void SourceTxtView::addTreeItemVars(HTREEITEM treeItem,
                                const GRDI::VariableValueMap& vars)
{
  m_itemVarMap[treeItem] = vars;
}
//=============================================
void SourceTxtView::addRefMap(const GRDI::LocationID& locID, const GRDI::LocationID& refID)
{
  m_refMap[findTreeItem(locID)] = refID;
}
//=============================================
void SourceTxtView::hideCursor()
{
 if (m_prevIt)
 {
    SrcItemType  type =  (SrcItemType) GetTreeCtrl().GetItemData(m_prevIt);
    type = SrcItemType(type & ~CURRENT);
    GetTreeCtrl().SetItemData(m_prevIt, type);
    setCallLocationImage(m_prevIt);
    m_prevIt = 0;
 }
}

//=============================================
void SourceTxtView::markItem(const GRDI::IGRDebuggerView::FuncLocationMarkDescrVec& markLocs)
{
  GRDI::IGRDebuggerView::FuncLocationMarkDescrVec::const_iterator it = markLocs.begin();
  for(; it != markLocs.end(); ++it)
  {
    const GRDI::LocationID& markLocID = it->m_loc.m_locID;
    const int& mark = it->m_mark;

    HTREEITEM it = findTreeItem(markLocID);
 
    SrcItemType  srcItemType  =  (SrcItemType) GetTreeCtrl().GetItemData(it);

    if (mark & GRDI::IGRDebuggerView::CURR)
    {
      hideCursor();
      srcItemType = (SrcItemType)(srcItemType | CURRENT);
      m_prevIt = it;
    }

    if (mark & GRDI::IGRDebuggerView::SET_BREAK)
    {
      if (!isBreakpoint(it))
      {
       m_breakpoints.insert(it);
       srcItemType = (SrcItemType)(srcItemType | BREAKPOINT);
      }
      else
      {
        m_breakpoints.erase(it);
        srcItemType = (SrcItemType)(srcItemType & ~BREAKPOINT);
      }
    } 
    if (mark & GRDI::IGRDebuggerView::HIT_BREAK)
        srcItemType = (SrcItemType)(srcItemType | BREAKPOINT);

    GetTreeCtrl().SetItemData(it, srcItemType);
    setCallLocationImage(it);
  // Ensure the item is visible.
    GetTreeCtrl().EnsureVisible(it);

  // Expand the parent, if possible.
    HTREEITEM hParent = GetTreeCtrl().GetParentItem(it);
    if (hParent != NULL)
    {
     expandRecUp(hParent );
    }
  } 
}
//=============================================
void SourceTxtView::expandRecUp(HTREEITEM hmyItem )
{
  if (hmyItem == NULL)
    return;
  HTREEITEM hChildItem = GetTreeCtrl().GetNextItem(hmyItem, TVGN_PARENT);
  expandRecUp( hChildItem);
  GetTreeCtrl().Expand(hmyItem, TVE_EXPAND);
  setCallLocationImage(hmyItem);
}
//=============================================
void SourceTxtView::selectLocation(const GRDI::LocationID& markLocID)
{
     
  HTREEITEM it = findTreeItem(markLocID);
 
  HTREEITEM hParent = GetTreeCtrl().GetParentItem(it);
  if (hParent != NULL)
  {
    expandRecUp(it);
    GetTreeCtrl().Select(hParent, TVGN_DROPHILITE);
  }
  Invalidate(); 
}

//=============================================
void SourceTxtView::clear()
{
  m_refMap.clear();
  m_idMap.clear();
  m_itemMap.clear();
  m_breakpoints.clear();
  m_itemVarMap.clear();
  m_prevIt = 0;
  GetTreeCtrl().DeleteAllItems();
}
/////////////////////////////////////////////////////////////////////////////
// SourceTxtView message handlers


//=============================================
void SourceTxtView::OnInitialUpdate() 
{
	CTreeView::OnInitialUpdate();
	
  m_imageList = new CImageList();
  m_imageList->Create(16, 16, ILC_COLOR16, 0, 4);

  CBitmap bm0;
  bm0.LoadBitmap(IDB_CALLOC_CLOSED);
  m_imageList->Add(&bm0, RGB(0, 0, 0));

  CBitmap bm1;
  bm1.LoadBitmap(IDB_CALLOC_OPEN);
  m_imageList->Add(&bm1, RGB(0, 0, 0));

  CBitmap bm2;
  bm2.LoadBitmap(IDB_CURRLOCATION);
  m_imageList->Add(&bm2, RGB(0, 0, 0));

  CBitmap bm3;
  bm3.LoadBitmap(IDB_BREAKPOINT);
  m_imageList->Add(&bm3, RGB(0, 0, 0));

  CBitmap bm4;
  bm4.LoadBitmap(IDB_CURR_BREAK);
  m_imageList->Add(&bm4, RGB(0, 0, 0));


  GetTreeCtrl().SetImageList(m_imageList, TVSIL_NORMAL);

}
//=============================================
bool SourceTxtView::isBreakpoint(HTREEITEM hItem) 
{
  ItemSet::const_iterator itB = m_breakpoints.find(hItem);
  if (itB == m_breakpoints.end())
    return false;
  else
    return true;
}
// //=============================================
// void SourceTxtView::toggleBreakPoint(HTREEITEM hItem) 
// {
// 
//   SrcItemType  type =  (SrcItemType) GetTreeCtrl().GetItemData(it);
//   if (!isBreakpoint(hItem))
//   {
//    m_breakpoints.insert(hItem);
//    type |= BREAKPOINT;
//   }
//   else
//   {
//     m_breakpoints.erase(hItem);
//     GetTreeCtrl().SetItemData(hItem & ~BREAKPOINT);
//     return false;
//   }
//   return false;
// }
//=============================================
void SourceTxtView::OnTogglebreakpoint() 
{
  ((CGRDebuggerGUIDoc*)GetDocument())->viewsChanged();
  HTREEITEM hItem = GetTreeCtrl().GetSelectedItem();

  if ((hItem != NULL) && (hItem != m_rootItem))
  {
     GetTreeCtrl().EnsureVisible(hItem);
     std::stringstream cmd;
     ItemLocIDMap::const_iterator it = m_itemMap.find(hItem );
     if (it != m_itemMap.end())
     {
      const GRDI::LocationID& id  = it->second;

      if(!isBreakpoint(hItem))
         cmd << "set "  << id;
       else
         cmd << "clear " << id;

       ((CGRDebuggerGUIDoc*)GetDocument())->debuggerCtrl(cmd.str());
     }
  }
}
//=============================================
void SourceTxtView::OnUpdateTogglebreakpoint(CCmdUI* pCmdUI) 
{
  HTREEITEM hItem = GetTreeCtrl().GetSelectedItem();

  if (((CGRDebuggerGUIDoc*)GetDocument())->m_ctrlEnabled 
        && m_idMap.size() &&
        (hItem != m_rootItem) &&
        (hItem != NULL) )
  {
    HTREEITEM hItem = GetTreeCtrl().GetSelectedItem();

    if ((hItem != NULL) && (hItem != m_rootItem) && isBreakpoint(hItem) )
            pCmdUI->SetCheck(1);
       else
            pCmdUI->SetCheck(0);
    pCmdUI->Enable( TRUE );
  }
  else
    pCmdUI->Enable( FALSE);
}
//=============================================
void SourceTxtView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
   HTREEITEM hItem = GetTreeCtrl().GetSelectedItem();
   if(hItem != NULL)
   {
     if (GetTreeCtrl().GetItemData(hItem) & REFERENCE )
     {
       ItemLocIDMap::const_iterator itI = m_refMap.find(hItem);
       if (itI != m_refMap.end())
       {
         const GRDI::LocationID& id = itI->second;
         const HTREEITEM newItem  = findTreeItem(itI->second);
         GetTreeCtrl().SelectItem(newItem);
         GetTreeCtrl().EnsureVisible(newItem);
         GetTreeCtrl().Expand(newItem, TVE_EXPAND);
         setCallLocationImage(newItem);
       }
     }
     else
     {
      GetTreeCtrl().Expand(hItem, TVE_TOGGLE);
      setCallLocationImage(hItem);
    }
   }
   //CTreeView::OnLButtonDblClk(nFlags, point);
}
//=============================================
void SourceTxtView::OnLog() 
{
   std::stringstream cmd;
   cmd << "log";
   if (m_log)
   {
     m_log = false;
     cmd << " off";
   }
   else
     m_log = true;
  ((CGRDebuggerGUIDoc*)GetDocument())->debuggerCtrl(cmd.str());
	
}

void SourceTxtView::OnUpdateLog(CCmdUI* pCmdUI) 
{
  if (((CGRDebuggerGUIDoc*)GetDocument())->m_ctrlEnabled && m_idMap.size())
  {
    if (m_log )
      pCmdUI->SetCheck(1);
    else
      pCmdUI->SetCheck(0);
    pCmdUI->Enable( TRUE );
  }
  else
    pCmdUI->Enable( FALSE );
}
//=============================================
void SourceTxtView::OnShowallreakpoints() 
{
  ((CGRDebuggerGUIDoc*)GetDocument())->debuggerCtrl("clear");
}

void SourceTxtView::OnUpdateShowallreakpoints(CCmdUI* pCmdUI) 
{
  if(((CGRDebuggerGUIDoc*)GetDocument())->m_ctrlEnabled && m_breakpoints.size())
	   pCmdUI->Enable( TRUE );
  else
    pCmdUI->Enable( FALSE );
}
//=============================================
void SourceTxtView::OnClearall() 
{
 ((CGRDebuggerGUIDoc*)GetDocument())->debuggerCtrl("clear all");

}

void SourceTxtView::OnUpdateClearall(CCmdUI* pCmdUI) 
{
  if(((CGRDebuggerGUIDoc*)GetDocument())->m_ctrlEnabled && m_breakpoints.size())
	  pCmdUI->Enable( TRUE );
  else
    pCmdUI->Enable( FALSE );
}

GRDI::IVariablePtr SourceTxtView::getVariable(int id) 
{

  IntIVariablePtrMap::const_iterator it = m_idVarMap.find(id);
  if (it != m_idVarMap.end())
    return it->second;
  
  return GRDI::IVariablePtr();
}

void SourceTxtView::OnRButtonDown(UINT nFlags, CPoint point) 
{
  UINT uFlags;
  HTREEITEM hItem = GetTreeCtrl().HitTest(point, &uFlags);

  if ((hItem != NULL) && (TVHT_ONITEM & uFlags))
  {
  

    ItemVarMap::const_iterator it = m_itemVarMap.find(hItem);
    if (it != m_itemVarMap.end() && !(it->second.empty()))
    {
      CMenu menu;
      VERIFY(menu.CreatePopupMenu());

      menu.AppendMenu(MF_BYPOSITION |  MF_STRING | MF_DISABLED, 0, "Variables");
      menu.AppendMenu(MF_SEPARATOR);

      m_idVarMap.clear();

      const GRDI::VariableValueMap& vars = it->second;
      GRDI::VariableValueMap::const_iterator itVars = vars.begin();
      int i = 0;
      for(;itVars != vars.end(); ++itVars, ++i)
      {
         GRDI::IVariable& varI = *(itVars->first);

         CBitmap* image = 0;
         switch (varI.getIVariableImpl()->m_type)
         {
           case GRDI::IVariableImpl::IO : 
            { 
              image = bm_inout;
              break;
            }
          case GRDI::IVariableImpl::INPUT :
            { 
              image = bm_in;
              break;
            }

          case GRDI::IVariableImpl::OUTPUT : 
            { 
              image = bm_out;
              break;
            }
          case GRDI::IVariableImpl::PATTERN : 
            { 
              image = bm_in;
              break;
            }
       }
       int id = ID_POPUP_DynCmd + i;

       
       menu.AppendMenu(MF_STRING, id, (LPCTSTR)itVars->first->name().c_str());
       menu.SetMenuItemBitmaps(id, MF_BYCOMMAND, image, image);

        m_idVarMap[id] = itVars->first;
      }


      CRect rect;
	    GetWindowRect (&rect);

	    menu.TrackPopupMenu (TPM_LEFTALIGN | TPM_RIGHTBUTTON,
		  		  rect.left + point.x,
			  	  //rect.top +::GetSystemMetrics(SM_CYCAPTION)+::GetSystemMetrics(SM_CYMENU) + point.y,
            rect.top + point.y,
				    this);
    }
  }
	
	CTreeView::OnRButtonDown(nFlags, point);
}

