// StackView.cpp : implementation file
//
#pragma warning(disable : 4786) 
#pragma warning(disable : 4503) 
#include "stdafx.h"
#include "GRDebuggerGUI.h"
#include "StackView.h"
#include "Messages.h"
#include "../GReATDebugger/GReATDebuggerVars.h"
#include <sstream>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// StackView

IMPLEMENT_DYNCREATE(StackView, CTreeView)

StackView::StackView() 
{
}

StackView::~StackView()
{
}


BEGIN_MESSAGE_MAP(StackView, CTreeView)
	//{{AFX_MSG_MAP(StackView)
	ON_WM_RBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
//===========================================
void StackView::clear()
{
  m_idMap.clear();
  m_varMap.clear();
  m_itemMap.clear();
  GetTreeCtrl().DeleteAllItems( );
}
//===========================================
void StackView::showEmptyText()
{
  if (!m_idMap.empty())
  {
    clear();
    GetTreeCtrl().InsertItem("no context", NULL);
  }
}

//===========================================
void StackView::deleteContext(const GRDI::ContextID& id)
{
  IdItemMap::iterator itId = m_idMap.find(id);
  if (itId != m_idMap.end())
  {
    GetTreeCtrl().DeleteItem(itId->second);
    m_idMap.erase(itId);
    m_itemMap.erase(itId->second);
  }

}
//===========================================
void StackView::collapse(const GRDI::ContextID& id)
{
  IdItemMap::const_iterator itId = m_idMap.find(id);
  if (itId != m_idMap.end())
  {
    GetTreeCtrl().Expand(itId->second, TVE_COLLAPSE );
  }
}
//===========================================
void StackView::addCtx(const GRDI::ContextDescr& contextDescr)
{
  std::stringstream idStr;
  idStr << contextDescr.m_id << " " << *(contextDescr.m_func);
  HTREEITEM ctxIt = m_idMap[contextDescr.m_id] = GetTreeCtrl().InsertItem(idStr.str().c_str(), NULL);
  const GRDI::LocationID& id = contextDescr.m_currLocationID;
  m_itemMap[ctxIt] = id;
  
  GetTreeCtrl().SetItemImage(ctxIt, 1, 1);  

  const GRDI::VariableValueMap& valMap = contextDescr.m_varValMap;
  GRDI::VariableValueMap::const_iterator it = valMap.begin(); 

   HTREEITEM inputIt = 0;
   HTREEITEM outputIt = 0;
   HTREEITEM ioIt = 0;
   HTREEITEM patternIt = 0;

 for (;it != valMap.end(); ++it)
 {
   GRDI::IVariable& varI = *(it->first);
  
   std::string varName = varI.name();

   HTREEITEM currIt = 0;
   std::string typ;
   switch (varI.getIVariableImpl()->m_type)
   {
      case GRDI::IVariableImpl::IO : 
        { 
          if (ioIt == 0)
          {
                ioIt = GetTreeCtrl().InsertItem("In/Out Variables", ctxIt );
                m_itemMap[ioIt] = id;
                GetTreeCtrl().SetItemImage(ioIt, 8, 8);  
          }
          currIt = GetTreeCtrl().InsertItem(varI.name().c_str(), ioIt  );
          break;
        }
      case GRDI::IVariableImpl::INPUT :
        { 
          if (inputIt == 0)
          {
                inputIt = GetTreeCtrl().InsertItem("In Variables", ctxIt );
                m_itemMap[inputIt] = id;
                GetTreeCtrl().SetItemImage(inputIt, 6, 6);  
          }
          currIt = GetTreeCtrl().InsertItem(varI.name().c_str(), inputIt );
          break;
        }

      case GRDI::IVariableImpl::OUTPUT : 
        { 
          if (outputIt == 0)
          {
                outputIt = GetTreeCtrl().InsertItem("Out Variables", ctxIt );
                m_itemMap[outputIt] = id;
                GetTreeCtrl().SetItemImage(outputIt, 7, 7);  
          }
          currIt = GetTreeCtrl().InsertItem(varI.name().c_str(), outputIt );
          break;
        }
      case GRDI::IVariableImpl::PATTERN : 
        { 
          if (patternIt == 0)
          {
                patternIt = GetTreeCtrl().InsertItem("Matchers", ctxIt );
                m_itemMap[patternIt] = id;
                GetTreeCtrl().SetItemImage(patternIt, 3, 3);  
          }
          currIt = GetTreeCtrl().InsertItem(varI.name().c_str(), patternIt);
          break;
        }
   }
   m_itemMap[currIt] = id;
   m_varMap[it->first] = currIt;
   GetTreeCtrl().SetItemImage(currIt , 5, 5);  
   GRDI::IVariableValuePtr varValI = it->second;
   if (varValI != GRDI::IVariableValuePtr())
   {
     const UdmObjectList& valList = varValI->getIVariableValueImpl()->getObjList();
   
     if (valList.size())
     {
        UdmObjectList::const_iterator itV = valList.begin();
        for (; itV != valList.end(); ++itV)
        {
          std::string val;
          getObjPath(*itV, val);
          HTREEITEM varValIt = GetTreeCtrl().InsertItem(val.c_str(), currIt );
          m_itemMap[varValIt] = id;
          GetTreeCtrl().SetItemImage(varValIt, 4, 4);  
        }
     }
   }
   else
   {
      HTREEITEM varValIt = GetTreeCtrl().InsertItem("has no value", currIt );
      m_itemMap[varValIt] = id;
      GetTreeCtrl().SetItemImage(varValIt, 4, 4);  
   }
 }
}
//===========================================
void StackView::showVar(const GRDI::IVariablePtr& var)
{
  VariablePtrItemMap::const_iterator it =  m_varMap.find(var);
  if (it !=  m_varMap.end())
  {
    const HTREEITEM& varIt = it->second;
    expandStackRecUp(varIt);
    expandStackRecDown(varIt);
    GetTreeCtrl().Select(varIt, TVGN_DROPHILITE);
  }
}
//===========================================
void StackView::expandStackRecDown(HTREEITEM hmyItem )
{
  if (hmyItem == NULL)
    return;
  HTREEITEM hChildItem = GetTreeCtrl().GetChildItem(hmyItem);
  while (hChildItem != NULL)
  {
      GetTreeCtrl().Expand(hChildItem, TVE_EXPAND);
      expandStackRecDown( hChildItem);
      hChildItem = GetTreeCtrl().GetNextItem(hChildItem, TVGN_NEXT);
  }
  GetTreeCtrl().Expand(hmyItem, TVE_EXPAND);

}

//===========================================
void StackView::expandStackRecUp(HTREEITEM hmyItem )
{
  if (hmyItem == NULL)
    return;
  HTREEITEM hChildItem = GetTreeCtrl().GetNextItem(hmyItem, TVGN_PARENT);
  expandStackRecUp( hChildItem);
  GetTreeCtrl().Expand(hmyItem, TVE_EXPAND);

}
//===========================================
void StackView::expandStack()
{
  if (m_idMap.size() == 0)
    return;
  IdItemMap::const_iterator itId = --m_idMap.end();
  expandStackRecDown( itId->second );
  GetTreeCtrl().SetItemImage(itId->second , 0, 0);  
}
//===========================================

/////////////////////////////////////////////////////////////////////////////
// StackView diagnostics

#ifdef _DEBUG
void StackView::AssertValid() const
{
	CTreeView::AssertValid();
}

void StackView::Dump(CDumpContext& dc) const
{
	CTreeView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// StackView message handlers


void StackView::OnInitialUpdate() 
{
	CTreeView::OnInitialUpdate();
	
	CTreeView::OnInitialUpdate();
	
  CImageList*  imageList = new CImageList();
  imageList->Create(16, 16, ILC_COLOR16, 0, 4);

  CBitmap bm0;
  bm0.LoadBitmap(IDB_CURRFRAME);
  imageList->Add(&bm0, RGB(0, 0, 0));

  CBitmap bm1;
  bm1.LoadBitmap(IDB_NONCURRFRAME);
  imageList->Add(&bm1, RGB(0, 0, 0));

  CBitmap bm2;
  bm2.LoadBitmap(IDB_VARIABLE);
  imageList->Add(&bm2, RGB(0, 0, 0));

  CBitmap bm3;
  bm3.LoadBitmap(IDB_VARIABLEGROUP);
  imageList->Add(&bm3, RGB(0, 0, 0));

  CBitmap bm4;
  bm4.LoadBitmap(IDB_VARIABLEVALUE);
  imageList->Add(&bm4, RGB(0, 0, 0));

  CBitmap bm5;
  bm5.LoadBitmap(IDB_VARIABLEINSTANCE);
  imageList->Add(&bm5, RGB(0, 0, 0));

  CBitmap bm6;
  bm6.LoadBitmap(IDB_LOCALVARIABLESIN);
  imageList->Add(&bm6, RGB(0, 0, 0));

  CBitmap bm7;
  bm7.LoadBitmap(IDB_LOCALVARIABLESOUT);
  imageList->Add(&bm7, RGB(0, 0, 0));

  CBitmap bm8;
  bm8.LoadBitmap(IDB_LOCALVARIABLESINOUT);
  imageList->Add(&bm8, RGB(0, 0, 0));

  GetTreeCtrl().SetImageList(imageList, TVSIL_NORMAL);
	
}

void StackView::OnRButtonDown(UINT nFlags, CPoint point) 
{
  HTREEITEM hItem = GetTreeCtrl().HitTest(point, &nFlags);

  if ((hItem != NULL) && (TVHT_ONITEM & nFlags))
  {
     ItemLocIDMap::const_iterator it = m_itemMap.find(hItem );
     if (it != m_itemMap.end())
     {
      CMenu menu;
      VERIFY(menu.CreatePopupMenu());
      m_showLocID = it->second;
      std::stringstream str;
      str << "Show the call location";
      menu.AppendMenu(MF_STRING, ID_POPUP_GOTO_ID, (LPCTSTR)str.str().c_str());

      CRect rect;
	    GetWindowRect (&rect);

	    menu.TrackPopupMenu (TPM_LEFTALIGN | TPM_RIGHTBUTTON,
		  		  rect.left + point.x,
            rect.top + point.y,
				    this);

     }
  }
	CTreeView::OnRButtonDown(nFlags, point);
}
