#include "TempDNProcessor.h"
#include "GReATUtils.h"

std::string getUdmProjectFolder( const UMLModelTransformer::Configuration& cfg)
{
	std::string upf;
	std::vector< UMLModelTransformer::MetaInformation> metas= cfg.MetaInformation_kind_children();
	if ( metas.empty())
		return upf;
	upf= getPath((metas.begin())->UdmProjectFile(), false);
	return upf;
}

///////////////////////////////////////////////////////////////////////////////
void TempDNProcessor::processPackages( const std::vector< UMLModelTransformer::Package>& ps, GReATConfig::UsageInformation& ui) const
{
	for( std::vector< UMLModelTransformer::Package>::const_iterator i= ps.begin(); i!= ps.end(); ++i) {
		const UMLModelTransformer::Package& currPackage= *i;
		if ( false == currPackage.temporary()) {
			continue;	// skip non-temporary packages
		}
		
		// !!VIZA this feature is not supported in GReAT 1.3.2
		//throw TempDNEx( "Warning: Temporary packages are not supported yet.\nTemporary attribute settings will be ignored.\n");

		UMLModelTransformer::Class root= findRootObjectInPackage( currPackage);
		if ( !root) {
			throw TempDNEx( "No root object found in package: " + ( std::string) currPackage.name());
		}
		std::string metaname= currPackage.name();
		GReATConfig::TemporaryFileUsage tfu= GReATConfig::TemporaryFileUsage::Create( ui);
		tfu.MetaName()= metaname;
		tfu.RootClassName()= root.name();
		tfu.DTDFileName()= _udmProjectFolder+ metaname+ ".xsd";
	}
}

UMLModelTransformer::Class TempDNProcessor::findRootObjectInPackage( const UMLModelTransformer::Package& p) const
{
	UMLModelTransformer::Class root;
	// root object is a class which is not contained in anything and 
	// neither is referenced by a reference which is contained in something.
	const std::vector< UMLModelTransformer::ClassDiagram>& cds= p.ClassDiagram_kind_children();
	for( std::vector< UMLModelTransformer::ClassDiagram>::const_iterator i= cds.begin(); i!= cds.end(); ++i) {
		const UMLModelTransformer::ClassDiagram& currCD= *i;
		const std::vector< UMLModelTransformer::Class>& classes= currCD.Class_kind_children();
		for( std::vector< UMLModelTransformer::Class>::const_iterator j= classes.begin(); j!= classes.end(); ++j) {
			const UMLModelTransformer::Class& currC= *j;
			const std::set< UMLModelTransformer::Composition>& parents= currC.dstParent();	// has parent?
			// if has not, and not referenced by a child, then it is root.
			if ( parents.empty() && !isReferencedByChild( currC)) { 
				return root= currC;
			}
		}
	}
	return root;
}

bool TempDNProcessor::isReferencedByChild( const UMLModelTransformer::Class& c) const
{
	const std::set< UMLModelTransformer::ClassCopy>& refs= c.src();
	for( std::set< UMLModelTransformer::ClassCopy>::const_iterator i= refs.begin(); i!= refs.end(); ++i) {
			const UMLModelTransformer::ClassCopy& currCC= *i;
			const std::set< UMLModelTransformer::Composition>& parents= currCC.dstParent();	// has parent?
			// if has, then 'c' is referenced by a child
			if ( false == parents.empty()) {
				return true;
			}
	}
	return false;	// not such a reference found.
}
