/*
 *	Copyright (c) Vanderbilt University, 2001-2002
 *	ALL RIGHTS RESERVED
 *
 *	Vanderbilt University disclaims all warranties with regard to this
 *	software, including all implied warranties of merchantability
 *	and fitness.  In no event shall Vanderbilt University be liable for
 *	any special, indirect or consequential damages or any damages
 *	whatsoever resulting from loss of use, data or profits, whether
 *	in an action of contract, negligence or other tortious action,
 *	arising out of or in connection with the use or performance of
 *	this software.
 *
 *  FILE NAME:	LibraryUpdate
 *	SYNOPSIS:	Class that updates Library files 
 *  AUTHOR:		Aditya Agrawal - aditya,agrawal@vanderbilt.edu
 *  CREATED:	08/24/03
 *  LAST UPDATE: 
 */

//#include "stdafx.h"
//#include <afxdlgs.h> // For CFileDialog

#include "LibraryUpdate.h"
//#include "algorithm"
//#include "cctype"

void LibraryUpdate::update()
{	
	cout << endl;
	cout << "Phase 1 - Building maps .... " << endl;
	Phase1_BuildMaps();
	cout << "Phase 2 - Transfering reference .... " << endl;
	Phase2_TransferReferences();
	cout << "Done." << endl;
}

void LibraryUpdate::Phase1_BuildMaps()
{	
	BuildClassMap(oldLib, oldClassMap);
	BuildOldNewMap(newLib, oldClassMap, newClassMap);
}

void LibraryUpdate::BuildClassMap(UMTContainer library, String2ClassMap &mmap)
{	vector<UMLModelTransformer::ClassDiagram> cds;
	if((string)library.obj.type().name() == "RootFolder")
		cds = UMLModelTransformer::RootFolder::Cast(library.obj).ClassDiagram_kind_children();
	else {
		cds = UMLModelTransformer::Package::Cast(library.obj).ClassDiagram_kind_children();
		vector<UMLModelTransformer::Namespace> nss = UMLModelTransformer::Package::Cast(library.obj).Namespace_kind_children();
		if(!nss.empty()) {
			vector<UMLModelTransformer::ClassDiagram> t = GetClassDiagrams( nss );
			cds.insert( cds.end(), t.begin(), t.end() );
		}
	}
	for(vector<UMLModelTransformer::ClassDiagram>::iterator ci=cds.begin(); ci!=cds.end();ci++)
	{	UMLModelTransformer::ClassDiagram cd = *ci;
		vector<UMLModelTransformer::Class> cls = cd.Class_kind_children();
		for(vector<UMLModelTransformer::Class>::iterator c=cls.begin(); c!=cls.end();c++)
		{	UMLModelTransformer::Class cl = *c;
			string name = GetQualifiedName(cl); //cl.name();
			if(mmap.find(name) !=mmap.end())
			{	string msg = "Error: In library: "+(string)library.name()+" more than one class with name: "+name;
				throw udm_exception(msg);
			}
			mmap[name] = cl;
		}
	}
}

void LibraryUpdate::BuildOldNewMap(UMTContainer library, String2ClassMap &oldM, Class2ClassMap &newM)
{	
	vector<UMLModelTransformer::ClassDiagram> cds;
	if((string)library.obj.type().name() == "RootFolder")
		cds = UMLModelTransformer::RootFolder::Cast(library.obj).ClassDiagram_kind_children();
	else {
		cds = UMLModelTransformer::Package::Cast(library.obj).ClassDiagram_kind_children();
		vector<UMLModelTransformer::Namespace> nss = UMLModelTransformer::Package::Cast(library.obj).Namespace_kind_children();
		if(!nss.empty()) {
			vector<UMLModelTransformer::ClassDiagram> t = GetClassDiagrams( nss );
			cds.insert( cds.end(), t.begin(), t.end() );
		}
	}
	for(vector<UMLModelTransformer::ClassDiagram>::iterator ci=cds.begin(); ci!=cds.end();ci++)
	{	UMLModelTransformer::ClassDiagram cd = *ci;
		vector<UMLModelTransformer::Class> cls = cd.Class_kind_children();
		for(vector<UMLModelTransformer::Class>::iterator c=cls.begin(); c!=cls.end();c++)
		{	UMLModelTransformer::Class cl = *c;
			string name = GetQualifiedName(cl); //cl.name();
			if(oldM.find(name) == oldM.end())
			{	continue;
			}
			UMLModelTransformer::Class oldC = oldM[name];
			newM[oldC] = cl;
		}
	}
}

void LibraryUpdate::Phase2_TransferReferences()
{	
	UMLModelTransformer::RootFolder rf = UMLModelTransformer::RootFolder::Cast(umt.GetRootObject());
	if(rf==Udm::null)
	{	throw udm_exception("Error: transformation doesnt have root folder");
	}
	
	vector<UMLModelTransformer::Transformation> trans = rf.Transformation_kind_children();
	for(vector<UMLModelTransformer::Transformation>::iterator i=trans.begin(); i!=trans.end(); i++)
	{	UMLModelTransformer::Transformation tra = *i;
		TransferReferences(tra);
	}

	vector<UMLModelTransformer::Package> packs = rf.Package_kind_children();
	for(vector<UMLModelTransformer::Package>::iterator j=packs.begin(); j!=packs.end(); j++)
	{	UMLModelTransformer::Package pack = *j;
		TransferReferences(pack);
	}
}

void LibraryUpdate::TransferReferences(UMLModelTransformer::Transformation tra)
{	vector<UMLModelTransformer::Expression> exps = tra.Expression_kind_children();
	for(vector<UMLModelTransformer::Expression>::iterator ei=exps.begin(); ei!=exps.end();ei++)
	{	UMLModelTransformer::Expression exp = *ei;
		TransferReferences(exp);
	}	
}

void LibraryUpdate::TransferReferences(UMLModelTransformer::Expression exp)
{	Uml::Class kind = exp.type();
	if(Uml::IsDerivedFrom(kind, UMLModelTransformer::CompoundRule::meta))
		TransferReferences(UMLModelTransformer::CompoundRule::Cast(exp));
	else if(Uml::IsDerivedFrom(kind, UMLModelTransformer::Test::meta))
		TransferReferences(UMLModelTransformer::Test::Cast(exp));
	else if(Uml::IsDerivedFrom(kind, UMLModelTransformer::PrimitiveRule::meta))
		TransferReferences(UMLModelTransformer::PrimitiveRule::Cast(exp));
	else
		throw udm_exception("Error: Expression is not anything");
}

void LibraryUpdate::TransferReferences(UMLModelTransformer::CompoundRule crule)
{	
	vector<UMLModelTransformer::Expression> exps = crule.Expression_kind_children();
	for(vector<UMLModelTransformer::Expression>::iterator ei=exps.begin(); ei!=exps.end();ei++)
	{	UMLModelTransformer::Expression exp = *ei;
		TransferReferences(exp);
	}
}

void LibraryUpdate::TransferReferences(UMLModelTransformer::Test test)
{	vector<UMLModelTransformer::Case> exps = test.Case_kind_children();
	for(vector<UMLModelTransformer::Case>::iterator ei=exps.begin(); ei!=exps.end();ei++)
	{	UMLModelTransformer::PrimitiveRule exp = *ei;
		TransferReferences(exp);
	}
}

void LibraryUpdate::TransferReferences(UMLModelTransformer::PrimitiveRule rule)
{	string rulename = rule.name();
	vector<UMLModelTransformer::PatternClass> pats = rule.PatternClass_kind_children();
	for(vector<UMLModelTransformer::PatternClass>::iterator ei=pats.begin(); ei!=pats.end();ei++)
	{	UMLModelTransformer::PatternClass pat = *ei;
		UMLModelTransformer::ClassBase cb = pat.ref();
		if(cb == Udm::null)
			continue;
		UMLModelTransformer::Class cl;
		if((string)cb.type().name() == "Class")
			cl = UMLModelTransformer::Class::Cast(cb);
		else if((string)cb.type().name() == "ClassCopy")
			cl = UMLModelTransformer::ClassCopy::Cast(cb).ref();
		else
		{	string error = "Error: in Rule:"+(string)rule.name()+ " PatternClass:"+ (string)pat.name() + " doesnt refer to Class or ClassCopy";
			throw udm_exception(error.c_str());
		}
		if(cl == Udm::null)
			continue;
		string name = GetQualifiedName(cl); //cl.name();
		if(oldClassMap.find(name) == oldClassMap.end())
			continue;
		if(newClassMap.find(cl) == newClassMap.end())
			continue;
		
		UMLModelTransformer::Class ncl = newClassMap[cl];
		
		pat.ref() = ncl;

	}
}

void LibraryUpdate::TransferReferences(UMLModelTransformer::Package pack)
{	vector<UMLModelTransformer::ClassDiagram> cds = pack.ClassDiagram_kind_children();
	for(vector<UMLModelTransformer::ClassDiagram>::iterator ei=cds.begin(); ei!=cds.end();ei++)
	{	UMLModelTransformer::ClassDiagram cd = *ei;
		TransferReferences(cd);
	}
}

void LibraryUpdate::TransferReferences(UMLModelTransformer::ClassDiagram cd)
{	vector<UMLModelTransformer::ClassCopy> cls = cd.ClassCopy_kind_children();
	for(vector<UMLModelTransformer::ClassCopy>::iterator ei=cls.begin(); ei!=cls.end();ei++)
	{	UMLModelTransformer::ClassCopy pat = *ei;
		UMLModelTransformer::ClassBase cb = pat.ref();
		UMLModelTransformer::Class cl;
		if((string)cb.type().name() == "Class")
			cl = UMLModelTransformer::Class::Cast(cb);
		else
			cl = UMLModelTransformer::ClassCopy::Cast(cb).ref();
		if(cl == Udm::null)
			continue;
		string name = GetQualifiedName(cl); //cl.name();
		if(oldClassMap.find(name) == oldClassMap.end())
			continue;
		if(newClassMap.find(cl) == newClassMap.end())
			continue;
		
		UMLModelTransformer::Class ncl = newClassMap[cl];
		
		pat.ref() = ncl;
		
	}
}


//Added for handling namespaces
vector<UMLModelTransformer::ClassDiagram> LibraryUpdate::GetClassDiagrams( vector<UMLModelTransformer::Namespace> &nss )
{
	vector<UMLModelTransformer::ClassDiagram> ret;
	for(vector<UMLModelTransformer::Namespace>::iterator ite = nss.begin(); ite != nss.end(); ite++) {
		vector<UMLModelTransformer::ClassDiagram> t = ite->ClassDiagram_kind_children();
		ret.insert( ret.end(), t.begin(), t.end() );

		vector<UMLModelTransformer::Namespace> morenss = ite->Namespace_kind_children();
		if(!morenss.empty()) {
			vector<UMLModelTransformer::ClassDiagram> t1 = GetClassDiagrams( morenss );
			ret.insert( ret.end(), t1.begin(), t1.end() );
		}
	}
	return ret;
}

string LibraryUpdate::GetQualifiedName(Udm::Object &o)
{
	string ret = o.getPath2("::");
	for(unsigned i=0;i<2;i++)
	{
		string::size_type pos = ret.find("::");
		if(pos != string::npos)
		{
			ret.erase(0, pos+2); // delete the begining along with the ::
		}
	}
	return ret;
}