///////////////////////////////////////////////////////////////////////////
// RawComponent.cpp, the main RAW COM component implementation file
// This is the file (along with its header RawComponent.h)
// that the component implementor is expected to modify in the first place
//
///////////////////////////////////////////////////////////////////////////
#include <iostream>
#include <fstream>

#include "stdafx.h"

#include "ComHelp.h"
#include "GMECOM.h"
#include <ComponentConfig.h>
#include "RawComponent.h"


CString progName;

// this method is called after all the generic initialization is done
// this should be empty, unless application-specific initialization is needed
STDMETHODIMP RawComponent::Initialize(struct IMgaProject *) {
	return S_OK;
}

// this is the obsolete component interface
// this present implementation either tries to call InvokeEx, or returns an error;
STDMETHODIMP RawComponent::Invoke(IMgaProject* gme, IMgaFCOs *models, long param) {
#ifdef SUPPORT_OLD_INVOKE
	CComPtr<IMgaFCO> focus;
	CComVariant parval = param;
	return InvokeEx(gme, focus, selected, parvar);
#else
	if(interactive) {
		AfxMessageBox("This component does not support the obsolete invoke mechanism");
	}
	return E_MGA_NOT_SUPPORTED;
#endif
}


// This is the main component method for interpereters and plugins. 
// May als be used in case of invokeable addons
STDMETHODIMP RawComponent::InvokeEx( IMgaProject *project,  IMgaFCO *currentobj,  
									IMgaFCOs *selectedobjs,  long param) {
	COMTRY {
	  if(interactive) {
		CComBSTR projname;
		CComBSTR focusname = "<nothing>";
		CComPtr<IMgaTerritory> terr;
		COMTHROW(project->CreateTerritory(NULL, &terr));
		COMTHROW(project->BeginTransaction(terr));
		try {
			COMTHROW(project->get_Name(&projname));
			if(currentobj) COMTHROW(currentobj->get_Name(&focusname));
			AfxMessageBox("RAW Com Component --- Plugin!!!! Sample (project: " + CString(projname) +
						", focus: " + CString(focusname));
			COMTHROW(project->CommitTransaction());
		}	catch(...) { project->AbortTransaction(); throw; }
		
	  } 
	} COMCATCH(;);
}

// GME currently does not use this function
// you only need to implement it if other invokation mechanisms are used
STDMETHODIMP RawComponent::ObjectsInvokeEx( IMgaProject *project,  IMgaObject *currentobj,  IMgaObjects *selectedobjs,  long param) {
	if(interactive) {
		AfxMessageBox("Tho ObjectsInvoke method is not implemented");
	}
	return E_MGA_NOT_SUPPORTED;
}


// implement application specific parameter-mechanism in these functions:
STDMETHODIMP RawComponent::get_ComponentParameter(BSTR name, VARIANT *pVal) {
	return S_OK;
}

STDMETHODIMP RawComponent::put_ComponentParameter(BSTR name, VARIANT newVal) {
	return S_OK;
}


#ifdef GME_ADDON

bool turnedon = true;
// these two functions are the main 
STDMETHODIMP RawComponent::GlobalEvent(globalevent_enum event) { 
	if(event == GLOBALEVENT_UNDO) 
	{	//AfxMessageBox("UNDO!!");
	}
	if( event == APPEVENT_XML_IMPORT_BEGIN)
		turnedon = false;
	else if( event == APPEVENT_XML_IMPORT_END)
		turnedon = true;

	return S_OK; 
}

STDMETHODIMP RawComponent::ObjectEvent(IMgaObject * obj, unsigned long eventmask, VARIANT v) 
{
	if ( !turnedon) return S_OK;

	if(eventmask & OBJEVENT_CREATED) 
	{	CComBSTR objID;
		COMTHROW(obj->get_ID(&objID));
		//AfxMessageBox( "Object created! ObjID: " + CString(objID));
		objtype_enum type;
		COMVERIFY(obj->get_ObjType(&type));
		if(type == OBJTYPE_CONNECTION) 
		{	
			CComQIPtr<IMgaConnection> ciConnection(obj);
			CString skname = GetConnectionKindName(ciConnection);
			if(skname == "PatternComposition" || skname == "PatternAssociation")
			{	// Get the objects from references and see if this association can exist
				SetAttribute(CComQIPtr<IMgaSimpleConnection>(ciConnection));
			}
			else if(skname == "Binding" )
			{	ciConnection->put_RegistryValue((CBstrIn)"color", (CBstrIn)"0xAA00AA");
			}
			else if(skname == "AssociationClass" )
			{	CString src, dst;
				if(!GetSrcDstRolenames(CComQIPtr<IMgaSimpleConnection>(ciConnection)))
					return S_OK;
			}
			else if(skname == "Src" || skname == "Dst")
			{
				if(!GetSrcDstRolenamesEx(CComQIPtr<IMgaSimpleConnection>(ciConnection)))
					return S_OK;
			}
			else if(skname == "MoveCopy" )
			{	
				SetMoveCopyConnAttribute(CComQIPtr<IMgaSimpleConnection>(ciConnection));
			}
		}
		if(type == OBJTYPE_SET) 
		{	
			CComQIPtr<IMgaSet> ciSet(obj);
			CBstr bstr;
			{	CComPtr<IMgaMetaFCO> meta;
				COMVERIFY(ciSet->get_Meta(&meta));
				COMVERIFY(meta->get_Name(bstr));
			}
			CString skname(bstr);
			if(skname == "Group")
			{
				CString criterial("/* Add your code with UDM APIs below to determine whether the two matches ");
				criterial.Append("\r\nshould be grouped together. Use prefix \"the_\" and \"other_\" with the ");
				criterial.Append("\r\nPatternClass name to represent the bound objects of two matches. For example:");
				criterial.Append("\r\nthe_ClassA, other_ClassA */");
				criterial.Append("\r\nreturn true;");	
				if( !SUCCEEDED( ciSet->put_StrAttrByName((CBstrIn)"Criterial", (CBstrIn)criterial) ) )
					return S_OK;
			}
		}
	}
	else if(eventmask & OBJEVENT_CONNECTED)
	{	CComBSTR objID;
		COMTHROW(obj->get_ID(&objID));
		//	AfxMessageBox( "Object created! ObjID: " + CString(objID));
		objtype_enum type;
		COMVERIFY(obj->get_ObjType(&type));
		if(type == OBJTYPE_ATOM) 
		{	CComQIPtr<IMgaAtom> ciAtom(obj);
			CBstr AtomName;
			COMVERIFY(ciAtom->get_Name(AtomName));
			//	AfxMessageBox(CString(AtomName));
		}
	}
	else if(eventmask & OBJEVENT_ATTR)
	{	
		CComBSTR objID;
		COMTHROW(obj->get_ID(&objID));
		//AfxMessageBox( "Object created! ObjID: " + CString(objID));
		objtype_enum type;
		COMVERIFY(obj->get_ObjType(&type));
		if(type == OBJTYPE_CONNECTION) 
		{	
			CComQIPtr<IMgaSimpleConnection> ciConnection(obj);
			CBstr bstr;
			{	CComPtr<IMgaMetaFCO> meta;
				COMVERIFY(ciConnection->get_Meta(&meta));
				COMVERIFY(meta->get_Name(bstr));
			}
			CString skname(bstr);
			CBstr connAction;
			

			if(skname == "PatternComposition" || skname == "PatternAssociation")
			{	if ( ! SUCCEEDED( ciConnection->get_StrAttrByName((CBstrIn)"Action", connAction) ) )
					return S_OK;

				if(CString(connAction) == "CreateNew")

				{	ciConnection->put_RegistryValue((CBstrIn)"color", (CBstrIn)"0x0000ff");
				}

				else if (CString(connAction) == "Delete")

				{	ciConnection->put_RegistryValue((CBstrIn)"color", (CBstrIn)"0xff0000");
				}
				else
				{	ciConnection->put_RegistryValue((CBstrIn)"color", (CBstrIn)"0x000000");
				}
			}
			if(skname == "PatternComposition")
			{	CBstr srcAction;
				
				CComPtr<IMgaFCO> iChildObject;
				COMVERIFY( ciConnection->get_Src(&iChildObject));
				if ( ! SUCCEEDED( iChildObject->get_StrAttrByName((CBstrIn)"Action", srcAction) ) )
					return S_OK;
				if(CString(connAction) != CString(srcAction))
					iChildObject->put_StrAttrByName((CBstrIn)"Action", (CBstrIn)connAction);
				return S_OK;
			}
		}
		else if(type == OBJTYPE_SET)
		{
			CComQIPtr<IMgaSet> ciSet(obj);
			CBstr bstr;
			{	CComPtr<IMgaMetaFCO> meta;
				COMVERIFY(ciSet->get_Meta(&meta));
				COMVERIFY(meta->get_Name(bstr));
			}
			CString skname(bstr);
			if(skname == "Group")
			{
				CBstr groupAction;
				if ( ! SUCCEEDED( ciSet->get_StrAttrByName((CBstrIn)"GroupAction", groupAction) ) )
					return S_OK;
				CString gAction(groupAction);
				if(gAction == "Move")
				{
					ciSet->put_RegistryValue((CBstrIn)"icon", (CBstrIn)"groupm.bmp");
				}
				else if(gAction == "Copy")
				{
					ciSet->put_RegistryValue((CBstrIn)"icon", (CBstrIn)"groupc.bmp");
				}
				else if(gAction == "Delete")
				{
					ciSet->put_RegistryValue((CBstrIn)"icon", (CBstrIn)"groupd.bmp");
				}
				else
				{
					ciSet->put_RegistryValue((CBstrIn)"icon", (CBstrIn)"groupb.bmp");
				}
				CComPtr<IMgaConnPoints> cpts;
				COMVERIFY( ciSet->get_PartOfConns(&cpts));
					
				MGACOLL_ITERATE(IMgaConnPoint, cpts) 
				{	CComQIPtr<IMgaConnection> ciConnection;
					COMVERIFY(MGACOLL_ITER->get_Owner(&ciConnection));
					SetMoveCopyConnAttribute(CComQIPtr<IMgaSimpleConnection>(ciConnection));			
				} MGACOLL_ITERATE_END;
			}
		}
		else if(type == OBJTYPE_REFERENCE) 
		{	
			CComQIPtr<IMgaReference> ciRef(obj);
			CBstr bstr;
			{	CComPtr<IMgaMetaFCO> meta;
				COMVERIFY(ciRef->get_Meta(&meta));
				COMVERIFY(meta->get_Name(bstr));
			}
			CString skname(bstr);

			if(skname == "PatternClass")
			{	CBstr connAction;
				if ( ! SUCCEEDED( ciRef->get_StrAttrByName((CBstrIn)"Action", connAction) ) )
					return S_OK;

				//if (CString(connAction) == "CreateNew" || CString(connAction) == "Delete")

				//{
					CComPtr<IMgaConnPoints> cpts;
					COMVERIFY( ciRef->get_PartOfConns(&cpts));
					
					MGACOLL_ITERATE(IMgaConnPoint, cpts) 
					{	CComQIPtr<IMgaConnection> ciConnection;
						COMVERIFY(MGACOLL_ITER->get_Owner(&ciConnection));
						CString skname = GetConnectionKindName(ciConnection);
						if(skname == "PatternComposition" || skname == "PatternAssociation")
						{	SetAttribute(CComQIPtr<IMgaSimpleConnection>(ciConnection));
						}
						
					} MGACOLL_ITERATE_END;
				//}
				
			}
		}
		/*else if(type == OBJTYPE_ATOM)
		{	CComQIPtr<IMgaAtom> ciAtom(obj);
			CBstr bstr;
			{	CComPtr<IMgaMetaFCO> meta;
				COMVERIFY(ciAtom->get_Meta(&meta));
				COMVERIFY(meta->get_Name(bstr));
			}
			CString skname(bstr);
			if(skname == "AttributeMapping")
			{	CBstr amStr;
				if ( ! SUCCEEDED( ciAtom->get_StrAttrByName((CBstrIn)"ExpressionString", amStr) ) )
					return S_OK;

				FIXME: use getGreatTempDir
				CString filename= getenv("GREAT_PATH");
				filename += "\\tmp\\zz_amtp.cpp";
				CString amexp(amStr);

				std::ofstream amout((LPCSTR)filename);
				amout << (LPCSTR)amexp;
				amout.close();
				
				progName = "\"C:\\Program Files\\Microsoft Visual Studio\\Common\\MSDev98\\Bin\\msdev.exe\"";
				CString systemcall = "call " + progName + " " + "\"" + filename + "\"";

				system((LPCSTR)systemcall);
				
				std::ifstream amin((LPCSTR)filename);
				
				
				CString aout;
				char ch[2];
				ch[1] = '\0';

				while(amin.get(ch[0]))	{	aout += ch; }
				amin.close();
					
				if(amexp == aout)
					return S_OK;

				if( !SUCCEEDED( ciAtom->put_StrAttrByName((CBstrIn)"ExpressionString", (CBstrIn)aout) ) )
					return S_OK;
			
			}

		}*/
	}
	
	return S_OK;
}

int RawComponent::SetAttribute(CComQIPtr<IMgaSimpleConnection> ciConnection)
{	CComPtr<IMgaFCO> isrc;
	CComPtr<IMgaFCO> idst;
	
	COMVERIFY(ciConnection->get_Src(&isrc) );
	CComQIPtr<IMgaReference> isrcAtom(isrc);

	COMVERIFY(ciConnection->get_Dst(&idst) );
	CComQIPtr<IMgaReference> idstAtom(idst);
					
	CBstr srcAction;
	if ( ! SUCCEEDED( isrcAtom->get_StrAttrByName((CBstrIn)"Action", srcAction) ) )
		return S_OK;
	CBstr dstAction;
	if ( ! SUCCEEDED( idstAtom->get_StrAttrByName((CBstrIn)"Action", dstAction) ) )
		return S_OK;
	CBstr connAction;
	if ( ! SUCCEEDED( ciConnection->get_StrAttrByName((CBstrIn)"Action", connAction) ) )
		return S_OK;

	if(CString(srcAction) == "CreateNew" && CString(connAction) != "CreateNew" || CString(dstAction) == "CreateNew" && CString(connAction) != "CreateNew")

	{	if( !SUCCEEDED( ciConnection->put_StrAttrByName((CBstrIn)"Action", (CBstrIn)"CreateNew") ) )
			return S_OK;
	}

	else if(CString(srcAction) == "Delete" && CString(connAction) != "Delete"|| CString(dstAction) == "Delete" && CString(connAction) != "Delete")
	{	if( !SUCCEEDED( ciConnection->put_StrAttrByName((CBstrIn)"Action", (CBstrIn)"Delete") ) )
			return S_OK;
	}
	
	CBstr bstr;
	{	CComPtr<IMgaMetaFCO> meta;
		COMVERIFY(ciConnection->get_Meta(&meta));
		COMVERIFY(meta->get_Name(bstr));
	}//couldn't use the GetConnectionKindName() function because ciConnection is the wrong type
	CString conntype = CString(bstr);
	//special PatternComposition stuff
	if(conntype == "PatternComposition")
	{
		if(CString(srcAction) == "Bound" && CString(dstAction) == "Bound") //parent and child "Bound"
			if( !SUCCEEDED( ciConnection->put_StrAttrByName((CBstrIn)"Action", (CBstrIn)"Bound") ) )
				return S_OK;
		if(CString(dstAction) == "Delete") //parent is Delete, all should be Delete
			if( !SUCCEEDED( ciConnection->put_StrAttrByName((CBstrIn)"Action", (CBstrIn)"Delete") ) )
				return S_OK;
		if(CString(dstAction) == "CreateNew") //parent is CreateNew, all should be CreateNew
			if( !SUCCEEDED( ciConnection->put_StrAttrByName((CBstrIn)"Action", (CBstrIn)"CreateNew") ) )
				return S_OK;
	}

	return S_OK;
}

int	RawComponent::SetMoveCopyConnAttribute(CComQIPtr<IMgaSimpleConnection> ciConnection)
{
	CComPtr<IMgaFCO> isrc;
	
	COMVERIFY(ciConnection->get_Src(&isrc) );
	CComQIPtr<IMgaSet> isrcSet(isrc);
					
	CBstr gAction;
	if ( ! SUCCEEDED( isrcSet->get_StrAttrByName((CBstrIn)"GroupAction", gAction) ) )
		return S_OK;

	if(CString(gAction) == "Move")
	{	
		ciConnection->put_RegistryValue((CBstrIn)"color", (CBstrIn)"0x009300");
		ciConnection->put_RegistryValue((CBstrIn)"lineType", (CBstrIn)"dash");
		ciConnection->put_RegistryValue((CBstrIn)"labelFormatStr", (CBstrIn)"move");
		ciConnection->put_RegistryValue((CBstrIn)"nameColor", (CBstrIn)"0x009300");
	}
	else if(CString(gAction) == "Copy")
	{
		ciConnection->put_RegistryValue((CBstrIn)"color", (CBstrIn)"0x880000");
		ciConnection->put_RegistryValue((CBstrIn)"lineType", (CBstrIn)"solid");
		ciConnection->put_RegistryValue((CBstrIn)"labelFormatStr", (CBstrIn)"copy");
		ciConnection->put_RegistryValue((CBstrIn)"nameColor", (CBstrIn)"0x880000");
	}
	else //do not create/delete the MoveCopy connection if the action is "Bound" or "Delete"
	{
		ciConnection->DestroyObject();
	}
	return S_OK;
}

CString RawComponent::GetConnectionKindName(CComQIPtr<IMgaConnection> ciConn)
{	CBstr bstr;
	{	CComPtr<IMgaMetaFCO> meta;
		COMVERIFY(ciConn->get_Meta(&meta));
		COMVERIFY(meta->get_Name(bstr));
	}
	return CString(bstr);
}

bool RawComponent::FixSrcDstRolenames(CComQIPtr<IMgaSimpleConnection> ciConn)
{
	//Do we need CComPtrs or CComQIPtrs?

	CComPtr<IMgaFCO> endOne, endTwo, iRealConn, iNObject;	//get the two ends of the connection
	CComPtr<IMgaFCO> kindOne, kindTwo;	//get the kinds of objects

	COMVERIFY(ciConn->get_Src(&endOne));
	COMVERIFY(ciConn->get_Dst(&endTwo));

	BSTR nameOne, nameTwo;			//get the names of the objects BSTR-style :)
	CString firstName, secondName;  //get the names of the objects CString-style :)

	//COMVERIFY(endOne->get_Name(&nameOne));
	//COMVERIFY(endTwo->get_Name(&nameTwo));
	//firstName = nameOne;
	//secondName = nameTwo;

	CComPtr<IMgaMetaFCO> firstType = NULL;	//try to get the types of the endpoints
	CComPtr<IMgaMetaFCO> secondType = NULL;

	COMVERIFY(endOne->get_Meta(&firstType));
	COMVERIFY(endTwo->get_Meta(&secondType));

	//AfxMessageBox("Before getting names");
	if(firstType != NULL) {
		nameOne = NULL;
		nameTwo = NULL;
		COMVERIFY(firstType->get_Name(&nameOne));
		COMVERIFY(secondType->get_Name(&nameTwo));
		//AfxMessageBox("After getting names");
		firstName = nameOne;
		secondName = nameTwo;
	}

	if(firstName == "Connector")
	{
		//AfxMessageBox("First pass, name = " + firstName);
		COMVERIFY(ciConn->get_Src(&iRealConn));
	}

	else if(secondName == "Connector")
	{
		//AfxMessageBox("Second pass, name = " + secondName);
		COMVERIFY(ciConn->get_Dst(&iRealConn));
	}
	
	else {
		return false;	//Nothing to do if no connector is involved
	}

	//mainConnector has the connector object, so now we see if there are any AssociationClass
	//connections coming from mainConnector; if so, then we can try to set the rolenames


	if(iRealConn)
	{
		//AfxMessageBox("one of them is a connector");

		//Here we are trying to make iRealConn into an IMgaSimpleConnection
		CComPtr<IMgaFCO> iObject;

		CComQIPtr<IMgaConnection> srcRoleConn, dstRoleConn;
		{
			CComPtr<IMgaConnPoints> opts;
			COMVERIFY( iRealConn->get_PartOfConns(&opts));	//iRealConn has the source of the connection
															//which in this case we know is a Connector object
							
			MGACOLL_ITERATE(IMgaConnPoint, opts)	//opts has the connection points
			{	
				CComQIPtr<IMgaConnection> cioConnection;
				COMVERIFY(MGACOLL_ITER->get_Owner(&cioConnection));
				CString sokname = GetConnectionKindName(cioConnection);
				/*
				if(sokname == "Src")
				{	srcRoleConn = cioConnection;
				}
				else if(sokname == "Dst")
				{	dstRoleConn = cioConnection;
				}*/
				if(sokname == "AssociationClass")
				{
					//AfxMessageBox("We have an ASSOCIATION CLASS, we can continue.");

					CComQIPtr<IMgaSimpleConnection> mainConnectorSimple;
					mainConnectorSimple = CComQIPtr<IMgaSimpleConnection>(cioConnection);

					COMVERIFY( mainConnectorSimple->get_Dst(&iObject));
					CComBSTR nameOfStuff;
					COMTHROW(iObject->get_Name(&nameOfStuff));
					//AfxMessageBox( "Working: " + CString(nameOfStuff));
					return GetSrcDstRolenames(mainConnectorSimple);
				}

				
			} MGACOLL_ITERATE_END;
		}

	}

	return true;
}


bool RawComponent::GetSrcDstRolenames(CComQIPtr<IMgaSimpleConnection> ciConn)
{
	CString name;
	CBstr srcRolename, dstRolename;

	CComPtr<IMgaModel> parent;			//these two lines get the parent of the connection
	ciConn->get_ParentModel(&parent);	//(name of the rule is the parent)
	
	CComPtr<IMgaFCO> iObject,iNObject,iRealConn;

	//These next two lines get destination and source of the connection and make sure that
	//the HRESULTS returned by each are non-negative (negative indicates failure)

	COMVERIFY( ciConn->get_Dst(&iObject));
	COMVERIFY( ciConn->get_Src(&iRealConn));

	//This loop stores the two connections in srcRoleConn and dstRoleConn

	CComQIPtr<IMgaConnection> srcRoleConn, dstRoleConn;
	{	CComPtr<IMgaConnPoints> opts;
		COMVERIFY( iRealConn->get_PartOfConns(&opts)); //iRealConn has the source of the connection
						
		MGACOLL_ITERATE(IMgaConnPoint, opts)	//opts has the connection points
		{	CComQIPtr<IMgaConnection> cioConnection;
			COMVERIFY(MGACOLL_ITER->get_Owner(&cioConnection));
			CString sokname = GetConnectionKindName(cioConnection);
			if(sokname == "Src")
			{	srcRoleConn = cioConnection;
			}
			else if(sokname == "Dst")
			{	dstRoleConn = cioConnection;
			}
			
		} MGACOLL_ITERATE_END;
	}
		
	objtype_enum type;
	COMVERIFY(iObject->get_ObjType(&type));		//iObject has the destination of the connection

	//This while loop continues until iObject is no longer a reference

	CComQIPtr<IMgaReference> iRef = iObject;
	while(type == OBJTYPE_REFERENCE) 
	{	
		
		CBstr refName;
		COMVERIFY(iRef->get_Name(refName));
		CString rName(refName);
		iNObject = NULL;

		COMVERIFY(iRef->get_Referred(&iNObject));
		iObject = iNObject;
		// VIZA fix null ref association class bug
		if ( iObject) {
			COMVERIFY(iObject->get_ObjType(&type));
			iRef = iObject;
		} else {
			break;
		}
	}
	if ( iObject) {
	
		CComQIPtr<IMgaAtom> iClass(iObject);	//iClass is an MgaAtom ptr to iObject
		
		// For Debug Purpose
		{	CBstr bstr;
			COMVERIFY(iClass->get_Name(bstr));
			name= bstr;
		}

		CComPtr<IMgaFCOs> iClassRefs;
		iClass->get_ReferencedBy(&iClassRefs);

		iClassRefs->Append(iClass);

		MGACOLL_ITERATE(IMgaFCO, iClassRefs)	//iClassRefs has the FCOs of the parent model
		{	
			CComPtr<IMgaFCO> iClassObj(MGACOLL_ITER);  //get the current MgaFCO and put it in iClassObj

			CComPtr<IMgaMetaFCO> meta;
			CBstr cobsName;
			COMVERIFY(iClassObj->get_Meta(&meta));  //meta has iClassObj's meta
			COMVERIFY(meta->get_Name(cobsName));
			CString cobjName(cobsName);

			if(CString(cobsName) != "ClassCopy" && CString(cobjName) != "Class")
				continue;
			
			CComPtr<IMgaConnPoints> cpts;
			COMVERIFY( iClassObj->get_PartOfConns(&cpts));	//cpts has the ends of the connection
							
			MGACOLL_ITERATE(IMgaConnPoint, cpts)	//iterate over the ends of the connetion
			{	CComQIPtr<IMgaConnection> ciConnts;
				COMVERIFY(MGACOLL_ITER->get_Owner(&ciConnts));		//ciConnts has the connection
				CString skname = GetConnectionKindName(ciConnts);

				while(skname == "Sub") //INHERITENCE - travel up till we find the AssociationClass
				{
					skname = ""; //to prevent infinite loop if the inheritence triangle isn't connected properly

					CComQIPtr<IMgaSimpleConnection> iSub = ciConnts;
					CComPtr<IMgaFCO> IInheritence; //the inheritence triangle
					COMVERIFY(iSub->get_Src(&IInheritence));

					CComPtr<IMgaConnPoints> cptsInheritence;
					COMVERIFY( IInheritence->get_PartOfConns(&cptsInheritence));
					MGACOLL_ITERATE(IMgaConnPoint, cptsInheritence)
					{
						CComQIPtr<IMgaConnection> ciConntsInheritence;
						COMVERIFY(MGACOLL_ITER->get_Owner(&ciConntsInheritence));
						CString sknameInheritence = GetConnectionKindName(ciConntsInheritence);
						
						if(sknameInheritence == "Base")
						{
							CComQIPtr<IMgaSimpleConnection> iBase = ciConntsInheritence;
							CComPtr<IMgaFCO> IBaseClass; // the baseclass
							COMVERIFY(iBase->get_Src(&IBaseClass));

							CComPtr<IMgaConnPoints> cptsBase;
							COMVERIFY( IBaseClass->get_PartOfConns(&cptsBase));
							MGACOLL_ITERATE(IMgaConnPoint, cptsBase)
							{
								CComQIPtr<IMgaConnection> ciConntsBase;
								COMVERIFY(MGACOLL_ITER->get_Owner(&ciConntsBase));
								CString sknameBase = GetConnectionKindName(ciConntsBase);

								if(sknameBase == "Sub") 
								{ // if it's "Sub" the while loop will go up to the next baseclass
									skname = sknameBase;
									ciConnts = ciConntsBase;
								} 

								if(sknameBase == "AssociationClass")		//If ciConntsBase is an Association Class Connection
								{	CComPtr<IMgaFCO> iSrcFco;               //Get the rolenames
									CComQIPtr<IMgaSimpleConnection> ciConnection(ciConntsBase);
									COMVERIFY( ciConnection->get_Src(&iSrcFco));
									CComQIPtr<IMgaAtom> iConnector(iSrcFco);
					
									CComPtr<IMgaConnPoints> opts;
									COMVERIFY( iConnector->get_PartOfConns(&opts));
									
									MGACOLL_ITERATE(IMgaConnPoint, opts) 
									{	CComQIPtr<IMgaConnection> cioConnection;
										COMVERIFY(MGACOLL_ITER->get_Owner(&cioConnection));
										CString sokname = GetConnectionKindName(cioConnection);
										if(sokname == "Src")
										{	cioConnection->get_StrAttrByName((CBstrIn)"srcRolename", srcRolename); 
										}
										else if(sokname == "Dst")
										{	cioConnection->get_StrAttrByName((CBstrIn)"dstRolename", dstRolename);
										}
									} MGACOLL_ITERATE_END;
								}
							} MGACOLL_ITERATE_END;
						}
					} MGACOLL_ITERATE_END;
				}

				if(skname == "AssociationClass")		//If ciConnts is an Association Class Connection
				{	CComPtr<IMgaFCO> iSrcFco;           //get the rolenames
					CComQIPtr<IMgaSimpleConnection> ciConnection(ciConnts);
					COMVERIFY( ciConnection->get_Src(&iSrcFco));
					CComQIPtr<IMgaAtom> iConnector(iSrcFco);
					
					CComPtr<IMgaConnPoints> opts;
					COMVERIFY( iConnector->get_PartOfConns(&opts));
									
					MGACOLL_ITERATE(IMgaConnPoint, opts) 
					{	CComQIPtr<IMgaConnection> cioConnection;
						COMVERIFY(MGACOLL_ITER->get_Owner(&cioConnection));
						CString sokname = GetConnectionKindName(cioConnection);
						if(sokname == "Src")
						{	cioConnection->get_StrAttrByName((CBstrIn)"srcRolename", srcRolename); 
						}
						else if(sokname == "Dst")
						{	cioConnection->get_StrAttrByName((CBstrIn)"dstRolename", dstRolename);
						}
					} MGACOLL_ITERATE_END;
				}
			} MGACOLL_ITERATE_END;
		} MGACOLL_ITERATE_END;
	}// if ( iObject)
	
	int stophere= 0;
	if(srcRoleConn)	{

		if( CString(srcRolename) == "") {
			stophere++;
		} else {
			srcRoleConn->put_StrAttrByName((CBstrIn)"srcRolename", (CBstrIn) srcRolename);
		}
	}
	if(dstRoleConn) {
		if ( CString(dstRolename) == "") {
			stophere--;
		} else {
		dstRoleConn->put_StrAttrByName((CBstrIn)"dstRolename", (CBstrIn) dstRolename);
		}
	}
	return true;
}

bool RawComponent::GetSrcDstRolenamesEx(CComQIPtr<IMgaSimpleConnection> ciConn)
{
	//This function just finds the "AssociationClass" connection then passes it to GetSrcDstRolenames
	//so that the rolenames will be filled in no matter what order the connections were made.
	CComPtr<IMgaFCO> iConnector;

	CBstr bstr;
	{	CComPtr<IMgaMetaFCO> meta;
		COMVERIFY(ciConn->get_Meta(&meta));
		COMVERIFY(meta->get_Name(bstr));
	}//couldn't use the GetConnectionKindName() function because ciConn is the wrong type
	CString conntype = CString(bstr);

	if(conntype == "Src")
	{
		COMVERIFY( ciConn->get_Dst(&iConnector));
	}
	if(conntype == "Dst")
	{
		COMVERIFY( ciConn->get_Src(&iConnector));
	}

	if(!iConnector)
	{
		return false; //shouldn't ever happen
	}

	CComQIPtr<IMgaConnection> AsscClassConn;
	{	
		CComPtr<IMgaConnPoints> opts;
		COMVERIFY( iConnector->get_PartOfConns(&opts)); //iConnector is the black dot in the middle of the 3-way connection
						
		MGACOLL_ITERATE(IMgaConnPoint, opts)	//opts has the connection points
		{	
			CComQIPtr<IMgaConnection> cioConnection;
			COMVERIFY(MGACOLL_ITER->get_Owner(&cioConnection));
			CString sokname = GetConnectionKindName(cioConnection);
			if(sokname == "AssociationClass")
			{	
				AsscClassConn = cioConnection;
			}
		} MGACOLL_ITERATE_END;
	}

	if(!AsscClassConn)
		return true; //it's ok for this to happen, just means the "AssociationClass" doesn't exist yet
	CComQIPtr<IMgaSimpleConnection> iConn = AsscClassConn;
	return GetSrcDstRolenames(iConn);
}


#endif
