#include <Include/PlatformDefinitions.hpp>
//XALAN_USING_STD(cerr)
//XALAN_USING_STD(cout)
//XALAN_USING_STD(endl)


//XALAN_USING_XERCES(XMLPlatformUtils)
//XALAN_USING_XALAN(XalanTransformer)

#include <windows.h>
#include <iostream>
#include <sstream>
#include <string>
#include <xercesc/util/PlatformUtils.hpp>
#include <XalanTransformer/XalanTransformer.hpp>


XERCES_CPP_NAMESPACE_USE
XALAN_CPP_NAMESPACE_USE

#include "XSLTransformer.h"



bool loadXSLTScript( const char* module_name,
					 const char* resource_name,
					 const char* resource_type,
					 std::string& script
					 )
{
	HMODULE hDllModule= GetModuleHandle( module_name);
	if ( 0 == hDllModule)
		return false;
//	char temp[ 100]= {'\0'};
//	DWORD ret= GetModuleFileName( hDllModule, temp, 100);
	// Find Resource
	HRSRC hRes= FindResource( hDllModule, resource_name, resource_type);
	if ( 0 == hRes)
		return false;
	// Load Resource
	HGLOBAL hScript= LoadResource( hDllModule, hRes);
	if ( 0 == hScript)
		return false;
	// Lock Resource
	LPVOID pScript= LockResource( hScript);
	if ( 0 == pScript)
		return false;
	// get copy of the resource into 'script'
	std::istringstream script_stream( static_cast<char*>(pScript));
	script= script_stream.str();	
	return true;
}



// the interface of this function is deliberately const char*.
// Strings are not permitted due to Xalan STL incompetency problems. 
int executeTransform( 
					 const char* file_in, 
					 const char* file_out, 
					 const char* module_name,
					 const char* resource_name,
					 const char* resource_type
					 )
{
	int	theResult = -1;
	// load the XSLT script resource.
	std::string xslt_script;	
	bool canLoadScript= loadXSLTScript( module_name,  resource_name, resource_type, xslt_script);
	if ( false== canLoadScript)
	{
		return theResult;
	}
	try
	{
	//	XALAN_USING_XERCES(XMLPlatformUtils)
		XALAN_USING_XALAN(XalanTransformer)
		// Call the static initializer for Xerces.
		XMLPlatformUtils::Initialize();
		// Initialize Xalan.
		XalanTransformer::initialize();
		{
			// Create a XalanTransformer.
			XalanTransformer theXalanTransformer;
			// Create stream
			std::istringstream iss( xslt_script);
			XSLTInputSource xslt_is;
			xslt_is.setStream( &iss);
			// Perform the transformation
			theResult = theXalanTransformer.transform( 
				XSLTInputSource( file_in), 
				xslt_is,
				XSLTResultTarget( file_out)
				);
			
			if(theResult != 0)
			{
				std::cerr << "XSLT Transform Error: \n" << theXalanTransformer.getLastError()
					<< std::endl  << std::endl; 
			}
		}
		// Terminate Xalan...
		XalanTransformer::terminate();
		// Terminate Xerces...
		XMLPlatformUtils::Terminate();
		// Clean up the ICU, if it's integrated...
		XalanTransformer::ICUCleanUp();
	}
	catch(...)
	{
		return -1;
	}
	return theResult;
};

int executeTransform( const char * file_in, const char * file_out, const char * file_xsl)
{
	int	theResult = -1;

	try
	{
	//	XALAN_USING_XERCES(XMLPlatformUtils)
		XALAN_USING_XALAN(XalanTransformer)

		// Call the static initializer for Xerces.
		XMLPlatformUtils::Initialize();

		// Initialize Xalan.
		XalanTransformer::initialize();

		{
	//		int __in[8];
			// Create a XalanTransformer.
			XalanTransformer theXalanTransformer;
			
			
			theResult = theXalanTransformer.transform(file_in, file_xsl, file_out);
			

			if(theResult != 0)
			{
				std::cerr << "XSLT Transform Error: \n" << theXalanTransformer.getLastError()
					<< std::endl  << std::endl; 
			}
		}

		// Terminate Xalan...
		XalanTransformer::terminate();

		// Terminate Xerces...
		XMLPlatformUtils::Terminate();

		// Clean up the ICU, if it's integrated...
		XalanTransformer::ICUCleanUp();
	}
	catch(...)
	{
		return -1;
	}

	return theResult;
};
